@@ -11,33 +11,25 @@
 #include <arpa/inet.h>
 #include <libmm-glib.h>
 
-#include "nm-core-internal.h"
+#include "libnm-core-aux-intern/nm-libnm-core-utils.h"
+#include "libnm-core-intern/nm-core-internal.h"
 #include "NetworkManagerUtils.h"
 #include "devices/nm-device-private.h"
-#include "platform/nm-platform.h"
-#include "nm-ip4-config.h"
-#include "nm-ip6-config.h"
+#include "libnm-platform/nm-platform.h"
+#include "nm-l3-config-data.h"
 
 #define NM_MODEM_BROADBAND_MODEM "modem"
 
-static gboolean
-MODEM_CAPS_3GPP(MMModemCapability caps)
-{
-    G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-    /* MM_MODEM_CAPABILITY_LTE_ADVANCED is marked as deprecated since ModemManager 1.14.0.
-     *
-     * The flag probably was never used, it certainly isn't used since 1.14.0.
-     *
-     * Still, just to be sure, there is no harm in checking it here. Suppress the
-     * warning, it should have no bad effect.
-     */
-    return NM_FLAGS_ANY(caps,
-                        (MM_MODEM_CAPABILITY_GSM_UMTS | MM_MODEM_CAPABILITY_LTE
-                         | MM_MODEM_CAPABILITY_LTE_ADVANCED));
-    G_GNUC_END_IGNORE_DEPRECATIONS
-}
+#if !MM_CHECK_VERSION(1, 14, 0)
+#define MM_MODEM_CAPABILITY_5GNR ((MMModemCapability) (1 << 6))
+#endif
 
-#define MODEM_CAPS_3GPP2(caps) (caps & (MM_MODEM_CAPABILITY_CDMA_EVDO))
+#define MODEM_CAPS_3GPP(caps) \
+    NM_FLAGS_ANY(             \
+        caps,                 \
+        (MM_MODEM_CAPABILITY_GSM_UMTS | MM_MODEM_CAPABILITY_LTE | MM_MODEM_CAPABILITY_5GNR))
+
+#define MODEM_CAPS_3GPP2(caps) NM_FLAGS_ANY((caps), MM_MODEM_CAPABILITY_CDMA_EVDO)
 
 /* Maximum time to keep the DBus call waiting for a connection result.
  * This value is greater than the default timeout in ModemManager (180s since
@@ -61,13 +53,13 @@ typedef struct {
     ConnectStep       step;
 
     MMModemCapability          caps;
-    NMConnection *             connection;
-    GCancellable *             cancellable;
+    NMConnection              *connection;
+    GCancellable              *cancellable;
     MMSimpleConnectProperties *connect_properties;
-    GArray *                   ip_types;
+    GArray                    *ip_types;
     guint                      ip_types_i;
     guint                      ip_type_tries;
-    GError *                   first_error;
+    GError                    *first_error;
 } ConnectContext;
 
 /*****************************************************************************/
@@ -78,21 +70,18 @@ typedef struct {
     /* The modem object from dbus */
     MMObject *modem_object;
     /* Per-interface objects */
-    MMModem *      modem_iface;
-    MMModem3gpp *  modem_3gpp_iface;
+    MMModem       *modem_iface;
+    MMModem3gpp   *modem_3gpp_iface;
     MMModemSimple *simple_iface;
-    MMSim *        sim_iface;
+    MMSim         *sim_iface;
 
     /* Connection setup */
     ConnectContext *ctx;
 
-    MMBearer *        bearer;
+    MMBearer         *bearer;
     MMBearerIpConfig *ipv4_config;
     MMBearerIpConfig *ipv6_config;
 
-    guint idle_id_ip4;
-    guint idle_id_ip6;
-
     guint32 pin_tries;
 } NMModemBroadbandPrivate;
 
@@ -122,7 +111,7 @@ G_DEFINE_TYPE(NMModemBroadband, nm_modem_broadband, NM_TYPE_MODEM)
         if (nm_logging_enabled(_level, (_NMLOG_DOMAIN))) {                       \
             NMModemBroadband *const __self = (self);                             \
             char                    __prefix_name[128];                          \
-            const char *            __uid;                                       \
+            const char             *__uid;                                       \
                                                                                  \
             _nm_log(_level,                                                      \
                     (_NMLOG_DOMAIN),                                             \
@@ -198,24 +187,28 @@ translate_mm_error(NMModemBroadband *self, GError *error)
 /*****************************************************************************/
 
 static void
-get_capabilities(NMModem *                  _self,
+get_capabilities(NMModem                   *_self,
                  NMDeviceModemCapabilities *modem_caps,
                  NMDeviceModemCapabilities *current_caps)
 {
-    NMModemBroadband * self          = NM_MODEM_BROADBAND(_self);
-    MMModemCapability  all_supported = MM_MODEM_CAPABILITY_NONE;
-    MMModemCapability *supported;
-    guint              n_supported;
+    NMModemBroadband          *self          = NM_MODEM_BROADBAND(_self);
+    MMModemCapability          all_supported = MM_MODEM_CAPABILITY_NONE;
+    gs_free MMModemCapability *supported     = NULL;
+    guint                      n_supported;
+    guint                      i;
+
+    G_STATIC_ASSERT(MM_MODEM_CAPABILITY_POTS == (guint64) NM_DEVICE_MODEM_CAPABILITY_POTS);
+    G_STATIC_ASSERT(MM_MODEM_CAPABILITY_CDMA_EVDO
+                    == (guint64) NM_DEVICE_MODEM_CAPABILITY_CDMA_EVDO);
+    G_STATIC_ASSERT(MM_MODEM_CAPABILITY_GSM_UMTS == (guint64) NM_DEVICE_MODEM_CAPABILITY_GSM_UMTS);
+    G_STATIC_ASSERT(MM_MODEM_CAPABILITY_LTE == (guint64) NM_DEVICE_MODEM_CAPABILITY_LTE);
+    G_STATIC_ASSERT(MM_MODEM_CAPABILITY_5GNR == (guint64) NM_DEVICE_MODEM_CAPABILITY_5GNR);
 
     /* For now, we don't care about the capability combinations, just merge all
      * combinations in a single mask */
     if (mm_modem_get_supported_capabilities(self->_priv.modem_iface, &supported, &n_supported)) {
-        guint i;
-
         for (i = 0; i < n_supported; i++)
             all_supported |= supported[i];
-
-        g_free(supported);
     }
 
     *modem_caps = (NMDeviceModemCapabilities) all_supported;
@@ -226,7 +219,7 @@ get_capabilities(NMModem *                  _self,
 static gboolean
 owns_port(NMModem *_self, const char *iface)
 {
-    NMModemBroadband *     self    = NM_MODEM_BROADBAND(_self);
+    NMModemBroadband      *self    = NM_MODEM_BROADBAND(_self);
     const MMModemPortInfo *ports   = NULL;
     guint                  n_ports = 0, i;
 
@@ -277,7 +270,7 @@ create_cdma_connect_properties(NMConnection *connection)
 #if !MM_CHECK_VERSION(1, 9, 1)
     {
         NMSettingCdma *setting;
-        const char *   str;
+        const char    *str;
 
         setting = nm_connection_get_setting_cdma(connection);
         str     = nm_setting_cdma_get_number(setting);
@@ -291,14 +284,14 @@ create_cdma_connect_properties(NMConnection *connection)
 
 static MMSimpleConnectProperties *
 create_gsm_connect_properties(NMConnection *connection,
-                              const char *  apn,
-                              const char *  username,
-                              const char *  password)
+                              const char   *apn,
+                              const char   *username,
+                              const char   *password)
 {
-    NMSettingGsm *             setting;
-    NMSettingPpp *             s_ppp;
+    NMSettingGsm              *setting;
+    NMSettingPpp              *s_ppp;
     MMSimpleConnectProperties *properties;
-    const char *               str;
+    const char                *str;
 
     setting = nm_connection_get_setting_gsm(connection);
 
@@ -368,11 +361,11 @@ static void connect_context_step(NMModemBroadband *self);
 static void
 connect_ready(MMModemSimple *simple_iface, GAsyncResult *res, NMModemBroadband *self)
 {
-    ConnectContext *ctx;
-    GError *        error            = NULL;
-    NMModemIPMethod ip4_method       = NM_MODEM_IP_METHOD_UNKNOWN;
-    NMModemIPMethod ip6_method       = NM_MODEM_IP_METHOD_UNKNOWN;
-    gs_unref_object MMBearer *bearer = NULL;
+    ConnectContext           *ctx;
+    GError                   *error      = NULL;
+    NMModemIPMethod           ip4_method = NM_MODEM_IP_METHOD_UNKNOWN;
+    NMModemIPMethod           ip6_method = NM_MODEM_IP_METHOD_UNKNOWN;
+    gs_unref_object MMBearer *bearer     = NULL;
 
     bearer = mm_modem_simple_connect_finish(simple_iface, res, &error);
 
@@ -479,18 +472,18 @@ send_pin_ready(MMSim *sim, GAsyncResult *result, NMModemBroadband *self)
 }
 
 static void
-find_gsm_apn_cb(const char *  apn,
-                const char *  username,
-                const char *  password,
-                const char *  gateway,
-                const char *  auth_method,
+find_gsm_apn_cb(const char   *apn,
+                const char   *username,
+                const char   *password,
+                const char   *gateway,
+                const char   *auth_method,
                 const GSList *dns,
-                GError *      error,
+                GError       *error,
                 gpointer      user_data)
 {
-    NMModemBroadband *       self = user_data;
+    NMModemBroadband        *self = user_data;
     NMModemBroadbandPrivate *priv = NM_MODEM_BROADBAND_GET_PRIVATE(self);
-    ConnectContext *         ctx  = priv->ctx;
+    ConnectContext          *ctx  = priv->ctx;
 
     if (error) {
         _LOGW("failed to connect '%s': APN not found: %s",
@@ -513,14 +506,14 @@ static gboolean
 try_create_connect_properties(NMModemBroadband *self)
 {
     NMModemBroadbandPrivate *priv = NM_MODEM_BROADBAND_GET_PRIVATE(self);
-    ConnectContext *         ctx  = priv->ctx;
+    ConnectContext          *ctx  = priv->ctx;
 
     if (MODEM_CAPS_3GPP(ctx->caps)) {
         NMSettingGsm *s_gsm = nm_connection_get_setting_gsm(ctx->connection);
 
         if (!s_gsm || nm_setting_gsm_get_auto_config(s_gsm)) {
             gs_unref_object MMModem3gpp *modem_3gpp = NULL;
-            const char *                 network_id = NULL;
+            const char                  *network_id = NULL;
 
             s_gsm = nm_connection_get_setting_gsm(ctx->connection);
             if (s_gsm)
@@ -589,7 +582,7 @@ connect_context_step(NMModemBroadband *self)
         if (MODEM_CAPS_3GPP(ctx->caps)
             && mm_modem_get_unlock_required(self->_priv.modem_iface) == MM_MODEM_LOCK_SIM_PIN) {
             NMSettingGsm *s_gsm = nm_connection_get_setting_gsm(ctx->connection);
-            const char *  pin   = nm_setting_gsm_get_pin(s_gsm);
+            const char   *pin   = nm_setting_gsm_get_pin(s_gsm);
 
             /* If we have a PIN already, send it.  If we don't, get it. */
             if (pin) {
@@ -697,8 +690,8 @@ connect_context_step(NMModemBroadband *self)
 }
 
 static NMActStageReturn
-modem_act_stage1_prepare(NMModem *            _self,
-                         NMConnection *       connection,
+modem_act_stage1_prepare(NMModem             *_self,
+                         NMConnection        *connection,
                          NMDeviceStateReason *out_failure_reason)
 {
     NMModemBroadband *self = NM_MODEM_BROADBAND(_self);
@@ -771,15 +764,15 @@ check_connection_compatible_with_modem(NMModem *_self, NMConnection *connection,
 /*****************************************************************************/
 
 static gboolean
-complete_connection(NMModem *            modem,
-                    const char *         iface,
-                    NMConnection *       connection,
+complete_connection(NMModem             *modem,
+                    const char          *iface,
+                    NMConnection        *connection,
                     NMConnection *const *existing_connections,
-                    GError **            error)
+                    GError             **error)
 {
     NMModemBroadband *self = NM_MODEM_BROADBAND(modem);
     MMModemCapability modem_caps;
-    NMSettingPpp *    s_ppp;
+    NMSettingPpp     *s_ppp;
 
     modem_caps = mm_modem_get_current_capabilities(self->_priv.modem_iface);
 
@@ -829,11 +822,7 @@ complete_connection(NMModem *            modem,
     if (MODEM_CAPS_3GPP2(modem_caps)) {
         NMSettingCdma *s_cdma;
 
-        s_cdma = nm_connection_get_setting_cdma(connection);
-        if (!s_cdma) {
-            s_cdma = (NMSettingCdma *) nm_setting_cdma_new();
-            nm_connection_add_setting(connection, NM_SETTING(s_cdma));
-        }
+        s_cdma = _nm_connection_ensure_setting(connection, NM_TYPE_SETTING_CDMA);
 
         if (!nm_setting_cdma_get_number(s_cdma))
             g_object_set(G_OBJECT(s_cdma), NM_SETTING_CDMA_NUMBER, "#777", NULL);
@@ -863,7 +852,7 @@ complete_connection(NMModem *            modem,
 static gboolean
 get_user_pass(NMModem *modem, NMConnection *connection, const char **user, const char **pass)
 {
-    NMSettingGsm * s_gsm;
+    NMSettingGsm  *s_gsm;
     NMSettingCdma *s_cdma;
 
     s_gsm  = nm_connection_get_setting_gsm(connection);
@@ -961,259 +950,259 @@ set_mm_enabled(NMModem *_self, gboolean enabled)
 }
 
 /*****************************************************************************/
-/* IPv4 method static */
 
-static gboolean
-static_stage3_ip4_done(NMModemBroadband *self)
+static void
+stage3_ip_config_start(NMModem *modem, int addr_family, NMModemIPMethod ip_method)
 {
-    GError *        error               = NULL;
-    gs_unref_object NMIP4Config *config = NULL;
-    const char *                 data_port;
-    const char *                 address_string;
-    const char *                 gw_string;
-    guint32                      address_network;
-    guint32                      gw = 0;
-    NMPlatformIP4Address         address;
-    const char **                dns;
-    guint                        i;
-    guint32                      ip4_route_table, ip4_route_metric;
-    NMPlatformIP4Route *         r;
-    guint32                      mtu_n;
+    const int                               IS_IPv4 = NM_IS_IPv4(addr_family);
+    NMModemBroadband                       *self    = NM_MODEM_BROADBAND(modem);
+    nm_auto_unref_l3cd_init NML3ConfigData *l3cd    = NULL;
+    char                                    sbuf[sizeof(_nm_utils_to_string_buffer)];
+    gs_free_error GError                   *error = NULL;
+    const char                             *data_port;
+    const char                             *address_string;
+    const char                            **dns;
+    guint                                   i;
+    gboolean                                do_auto = FALSE;
+    int                                     ifindex;
+    NMUtilsIPv6IfaceId                      iid_data;
+    const NMUtilsIPv6IfaceId               *iid = NULL;
 
-    g_return_val_if_fail(self->_priv.ipv4_config, FALSE);
-    g_return_val_if_fail(self->_priv.bearer, FALSE);
+    if (IS_IPv4) {
+        g_return_if_fail(self->_priv.ipv4_config);
+        g_return_if_fail(self->_priv.bearer);
 
-    self->_priv.idle_id_ip4 = 0;
+        if (ip_method == NM_MODEM_IP_METHOD_AUTO) {
+            do_auto = TRUE;
+            goto out;
+        }
+    } else {
+        g_return_if_fail(self->_priv.ipv6_config);
+    }
 
-    _LOGI("IPv4 static configuration:");
+    if (IS_IPv4) {
+        guint32              address_network;
+        guint32              gw = 0;
+        NMPlatformIP4Address address;
+        NMPlatformIP4Route   route;
+        guint32              mtu_n;
+        const char          *gw_string;
 
-    /* Fully fail if invalid IP address retrieved */
-    address_string = mm_bearer_ip_config_get_address(self->_priv.ipv4_config);
-    if (!address_string
-        || !nm_utils_parse_inaddr_bin(AF_INET, address_string, NULL, &address_network)) {
-        error =
-            g_error_new(NM_DEVICE_ERROR,
+        _LOGI("IPv4 static configuration:");
+
+        /* Fully fail if invalid IP address retrieved */
+        address_string = mm_bearer_ip_config_get_address(self->_priv.ipv4_config);
+        if (!address_string
+            || !nm_utils_parse_inaddr_bin(AF_INET, address_string, NULL, &address_network)) {
+            g_set_error(&error,
+                        NM_DEVICE_ERROR,
                         NM_DEVICE_ERROR_INVALID_CONNECTION,
                         "(%s) retrieving IP4 configuration failed: invalid address given %s%s%s",
                         nm_modem_get_uid(NM_MODEM(self)),
                         NM_PRINT_FMT_QUOTE_STRING(address_string));
-        goto out;
-    }
+            goto out;
+        }
 
-    /* Missing gateway not a hard failure */
-    gw_string = mm_bearer_ip_config_get_gateway(self->_priv.ipv4_config);
-    if (gw_string && !nm_utils_parse_inaddr_bin(AF_INET, gw_string, NULL, &gw)) {
-        error =
-            g_error_new(NM_DEVICE_ERROR,
+        /* Missing gateway not a hard failure */
+        gw_string = mm_bearer_ip_config_get_gateway(self->_priv.ipv4_config);
+        if (gw_string && !nm_utils_parse_inaddr_bin(AF_INET, gw_string, NULL, &gw)) {
+            g_set_error(&error,
+                        NM_DEVICE_ERROR,
                         NM_DEVICE_ERROR_INVALID_CONNECTION,
                         "(%s) retrieving IP4 configuration failed: invalid gateway address \"%s\"",
                         nm_modem_get_uid(NM_MODEM(self)),
                         gw_string);
-        goto out;
-    }
+            goto out;
+        }
 
-    data_port = mm_bearer_get_interface(self->_priv.bearer);
-    g_return_val_if_fail(data_port, FALSE);
-    config = nm_ip4_config_new(nm_platform_get_multi_idx(NM_PLATFORM_GET),
-                               nm_platform_link_get_ifindex(NM_PLATFORM_GET, data_port));
+        data_port = mm_bearer_get_interface(self->_priv.bearer);
+        g_return_if_fail(data_port);
 
-    memset(&address, 0, sizeof(address));
-    address.address      = address_network;
-    address.peer_address = address_network;
-    address.plen         = mm_bearer_ip_config_get_prefix(self->_priv.ipv4_config);
-    address.addr_source  = NM_IP_CONFIG_SOURCE_WWAN;
-    if (address.plen <= 32)
-        nm_ip4_config_add_address(config, &address);
+        ifindex = nm_platform_link_get_ifindex(NM_PLATFORM_GET, data_port);
+        if (ifindex <= 0) {
+            g_set_error(&error,
+                        NM_DEVICE_ERROR,
+                        NM_DEVICE_ERROR_INVALID_CONNECTION,
+                        "(%s) data port %s not found",
+                        nm_modem_get_uid(NM_MODEM(self)),
+                        data_port);
+            goto out;
+        }
 
-    _LOGI("  address %s/%d", address_string, address.plen);
+        l3cd = nm_l3_config_data_new(nm_platform_get_multi_idx(NM_PLATFORM_GET),
+                                     ifindex,
+                                     NM_IP_CONFIG_SOURCE_WWAN);
+        nm_l3_config_data_set_dns_priority(l3cd, AF_INET, 0);
 
-    nm_modem_get_route_parameters(NM_MODEM(self), &ip4_route_table, &ip4_route_metric, NULL, NULL);
-    r = &(NMPlatformIP4Route){
-        .rt_source     = NM_IP_CONFIG_SOURCE_WWAN,
-        .gateway       = gw,
-        .table_coerced = nm_platform_route_table_coerce(ip4_route_table),
-        .metric        = ip4_route_metric,
-    };
-    nm_ip4_config_add_route(config, r, NULL);
-    _LOGI("  gateway %s", gw_string);
+        address = (NMPlatformIP4Address){
+            .address      = address_network,
+            .peer_address = address_network,
+            .plen         = mm_bearer_ip_config_get_prefix(self->_priv.ipv4_config),
+            .addr_source  = NM_IP_CONFIG_SOURCE_WWAN,
+        };
+        if (address.plen <= 32)
+            nm_l3_config_data_add_address_4(l3cd, &address);
 
-    /* DNS servers */
-    dns = mm_bearer_ip_config_get_dns(self->_priv.ipv4_config);
-    for (i = 0; dns && dns[i]; i++) {
-        if (nm_utils_parse_inaddr_bin(AF_INET, dns[i], NULL, &address_network)
-            && address_network > 0) {
-            nm_ip4_config_add_nameserver(config, address_network);
-            _LOGI("  DNS %s", dns[i]);
+        _LOGI("  address %s", nm_platform_ip4_address_to_string(&address, sbuf, sizeof(sbuf)));
+
+        route = (NMPlatformIP4Route){
+            .rt_source     = NM_IP_CONFIG_SOURCE_WWAN,
+            .gateway       = gw,
+            .table_any     = TRUE,
+            .table_coerced = 0,
+            .metric_any    = TRUE,
+            .metric        = 0,
+        };
+        nm_l3_config_data_add_route_4(l3cd, &route);
+        _LOGI("  gateway %s", gw_string);
+
+        dns = mm_bearer_ip_config_get_dns(self->_priv.ipv4_config);
+        for (i = 0; dns && dns[i]; i++) {
+            if (nm_utils_parse_inaddr_bin(AF_INET, dns[i], NULL, &address_network)
+                && address_network > 0) {
+                nm_l3_config_data_add_nameserver(l3cd, AF_INET, &address_network);
+                _LOGI("  DNS %s", dns[i]);
+            }
         }
-    }
 
 #if MM_CHECK_VERSION(1, 4, 0)
-    mtu_n = mm_bearer_ip_config_get_mtu(self->_priv.ipv4_config);
-    if (mtu_n) {
-        nm_ip4_config_set_mtu(config, mtu_n, NM_IP_CONFIG_SOURCE_WWAN);
-        _LOGI("  MTU %u", mtu_n);
-    }
-#endif
-
-out:
-    g_signal_emit_by_name(self, NM_MODEM_IP4_CONFIG_RESULT, config, error);
-    g_clear_error(&error);
-    return FALSE;
-}
-
-static NMActStageReturn
-static_stage3_ip4_config_start(NMModem *            modem,
-                               NMActRequest *       req,
-                               NMDeviceStateReason *out_failure_reason)
-{
-    NMModemBroadband *       self = NM_MODEM_BROADBAND(modem);
-    NMModemBroadbandPrivate *priv = NM_MODEM_BROADBAND_GET_PRIVATE(self);
-
-    /* We schedule it in an idle just to follow the same logic as in the
-     * generic modem implementation. */
-    nm_clear_g_source(&priv->idle_id_ip4);
-    priv->idle_id_ip4 = g_idle_add((GSourceFunc) static_stage3_ip4_done, self);
-
-    return NM_ACT_STAGE_RETURN_POSTPONE;
-}
-
-/*****************************************************************************/
-/* IPv6 method static */
-
-static gboolean
-stage3_ip6_done(NMModemBroadband *self)
-{
-    GError *             error  = NULL;
-    NMIP6Config *        config = NULL;
-    const char *         data_port;
-    const char *         address_string;
-    NMPlatformIP6Address address;
-    NMModemIPMethod      ip_method;
-    const char **        dns;
-    guint                i;
-
-    g_return_val_if_fail(self->_priv.ipv6_config, FALSE);
-
-    self->_priv.idle_id_ip6 = 0;
-    memset(&address, 0, sizeof(address));
-
-    ip_method = get_bearer_ip_method(self->_priv.ipv6_config);
-
-    address_string = mm_bearer_ip_config_get_address(self->_priv.ipv6_config);
-    if (!address_string) {
-        /* DHCP/SLAAC is allowed to skip addresses; other methods require it */
-        if (ip_method != NM_MODEM_IP_METHOD_AUTO) {
-            error = g_error_new(NM_DEVICE_ERROR,
-                                NM_DEVICE_ERROR_INVALID_CONNECTION,
-                                "(%s) retrieving IPv6 configuration failed: no address given",
-                                nm_modem_get_uid(NM_MODEM(self)));
+        mtu_n = mm_bearer_ip_config_get_mtu(self->_priv.ipv4_config);
+        if (mtu_n) {
+            nm_l3_config_data_set_mtu(l3cd, mtu_n);
+            _LOGI("  MTU %u", mtu_n);
         }
-        goto out;
-    }
+#endif
+    } else {
+        NMPlatformIP6Address address;
 
-    /* Fail if invalid IP address retrieved */
-    if (!inet_pton(AF_INET6, address_string, (void *) &(address.address))) {
-        error = g_error_new(NM_DEVICE_ERROR,
+        address_string = mm_bearer_ip_config_get_address(self->_priv.ipv6_config);
+        if (!address_string) {
+            /* DHCP/SLAAC is allowed to skip addresses; other methods require it */
+            if (ip_method != NM_MODEM_IP_METHOD_AUTO) {
+                g_set_error(&error,
+                            NM_DEVICE_ERROR,
                             NM_DEVICE_ERROR_INVALID_CONNECTION,
-                            "(%s) retrieving IPv6 configuration failed: invalid address given '%s'",
-                            nm_modem_get_uid(NM_MODEM(self)),
-                            address_string);
-        goto out;
-    }
+                            "(%s) retrieving IPv6 configuration failed: no address given",
+                            nm_modem_get_uid(NM_MODEM(self)));
+            }
+            goto out;
+        }
 
-    _LOGI("IPv6 base configuration:");
+        address = (NMPlatformIP6Address){};
 
-    data_port = mm_bearer_get_interface(self->_priv.bearer);
-    g_return_val_if_fail(data_port, FALSE);
+        if (!inet_pton(AF_INET6, address_string, &address.address)) {
+            g_set_error(&error,
+                        NM_DEVICE_ERROR,
+                        NM_DEVICE_ERROR_INVALID_CONNECTION,
+                        "(%s) retrieving IPv6 configuration failed: invalid address given '%s'",
+                        nm_modem_get_uid(NM_MODEM(self)),
+                        address_string);
+            goto out;
+        }
 
-    config = nm_ip6_config_new(nm_platform_get_multi_idx(NM_PLATFORM_GET),
-                               nm_platform_link_get_ifindex(NM_PLATFORM_GET, data_port));
+        data_port = mm_bearer_get_interface(self->_priv.bearer);
+        g_return_if_fail(data_port);
 
-    address.plen = mm_bearer_ip_config_get_prefix(self->_priv.ipv6_config);
-    if (address.plen <= 128)
-        nm_ip6_config_add_address(config, &address);
+        ifindex = nm_platform_link_get_ifindex(NM_PLATFORM_GET, data_port);
+        if (ifindex <= 0) {
+            g_set_error(&error,
+                        NM_DEVICE_ERROR,
+                        NM_DEVICE_ERROR_INVALID_CONNECTION,
+                        "(%s) data port %s not found",
+                        nm_modem_get_uid(NM_MODEM(self)),
+                        data_port);
+            goto out;
+        }
 
-    _LOGI("  address %s/%d", address_string, address.plen);
+        _LOGI("IPv6 base configuration:");
 
-    address_string = mm_bearer_ip_config_get_gateway(self->_priv.ipv6_config);
-    if (address_string) {
-        guint32 ip6_route_table, ip6_route_metric;
+        l3cd = nm_l3_config_data_new(nm_platform_get_multi_idx(NM_PLATFORM_GET),
+                                     ifindex,
+                                     NM_IP_CONFIG_SOURCE_WWAN);
+        nm_l3_config_data_set_dns_priority(l3cd, AF_INET6, 0);
 
-        if (inet_pton(AF_INET6, address_string, &address.address) != 1) {
-            error =
-                g_error_new(NM_DEVICE_ERROR,
+        do_auto = TRUE;
+
+        address.plen = mm_bearer_ip_config_get_prefix(self->_priv.ipv6_config);
+        if (address.plen <= 128) {
+            if (IN6_IS_ADDR_LINKLOCAL(&address.address)) {
+                iid_data.id = ((guint64 *) (&address.address.s6_addr))[1];
+                iid         = &iid_data;
+            } else
+                do_auto = FALSE;
+            nm_l3_config_data_add_address_6(l3cd, &address);
+        }
+
+        _LOGI("  address %s (slaac %s)",
+              nm_platform_ip6_address_to_string(&address, sbuf, sizeof(sbuf)),
+              do_auto ? "enabled" : "disabled");
+
+        address_string = mm_bearer_ip_config_get_gateway(self->_priv.ipv6_config);
+        if (address_string) {
+            if (inet_pton(AF_INET6, address_string, &address.address) != 1) {
+                g_set_error(&error,
+                            NM_DEVICE_ERROR,
                             NM_DEVICE_ERROR_INVALID_CONNECTION,
                             "(%s) retrieving IPv6 configuration failed: invalid gateway given '%s'",
                             nm_modem_get_uid(NM_MODEM(self)),
                             address_string);
+                goto out;
+            }
+
+            {
+                const NMPlatformIP6Route r = {
+                    .rt_source     = NM_IP_CONFIG_SOURCE_WWAN,
+                    .gateway       = address.address,
+                    .table_any     = TRUE,
+                    .table_coerced = 0,
+                    .metric_any    = TRUE,
+                    .metric        = 0,
+                };
+
+                _LOGI("  gateway %s", address_string);
+                nm_l3_config_data_add_route_6(l3cd, &r);
+            }
+        } else if (ip_method == NM_MODEM_IP_METHOD_STATIC) {
+            /* Gateway required for the 'static' method */
+            g_set_error(&error,
+                        NM_DEVICE_ERROR,
+                        NM_DEVICE_ERROR_INVALID_CONNECTION,
+                        "(%s) retrieving IPv6 configuration failed: missing gateway",
+                        nm_modem_get_uid(NM_MODEM(self)));
             goto out;
         }
 
-        nm_modem_get_route_parameters(NM_MODEM(self),
-                                      NULL,
-                                      NULL,
-                                      &ip6_route_table,
-                                      &ip6_route_metric);
-        {
-            const NMPlatformIP6Route r = {
-                .rt_source     = NM_IP_CONFIG_SOURCE_WWAN,
-                .gateway       = address.address,
-                .table_coerced = nm_platform_route_table_coerce(ip6_route_table),
-                .metric        = ip6_route_metric,
-            };
+        dns = mm_bearer_ip_config_get_dns(self->_priv.ipv6_config);
+        for (i = 0; dns && dns[i]; i++) {
+            struct in6_addr addr;
 
-            _LOGI("  gateway %s", address_string);
-            nm_ip6_config_add_route(config, &r, NULL);
-        }
-    } else if (ip_method == NM_MODEM_IP_METHOD_STATIC) {
-        /* Gateway required for the 'static' method */
-        error = g_error_new(NM_DEVICE_ERROR,
-                            NM_DEVICE_ERROR_INVALID_CONNECTION,
-                            "(%s) retrieving IPv6 configuration failed: missing gateway",
-                            nm_modem_get_uid(NM_MODEM(self)));
-        goto out;
-    }
-
-    /* DNS servers */
-    dns = mm_bearer_ip_config_get_dns(self->_priv.ipv6_config);
-    for (i = 0; dns && dns[i]; i++) {
-        struct in6_addr addr;
-
-        if (inet_pton(AF_INET6, dns[i], &addr)) {
-            nm_ip6_config_add_nameserver(config, &addr);
-            _LOGI("  DNS %s", dns[i]);
+            if (inet_pton(AF_INET6, dns[i], &addr)) {
+                nm_l3_config_data_add_nameserver(l3cd, AF_INET6, &addr);
+                _LOGI("  DNS %s", dns[i]);
+            }
         }
     }
 
 out:
-    nm_modem_emit_ip6_config_result(NM_MODEM(self), config, error);
-    g_clear_object(&config);
-    g_clear_error(&error);
-    return FALSE;
-}
+    if (error) {
+        nm_modem_emit_signal_new_config_failure(modem,
+                                                addr_family,
+                                                NM_DEVICE_STATE_REASON_IP_CONFIG_UNAVAILABLE,
+                                                error);
+        return;
+    }
 
-static NMActStageReturn
-stage3_ip6_config_request(NMModem *modem, NMDeviceStateReason *out_failure_reason)
-{
-    NMModemBroadband *       self = NM_MODEM_BROADBAND(modem);
-    NMModemBroadbandPrivate *priv = NM_MODEM_BROADBAND_GET_PRIVATE(self);
-
-    /* We schedule it in an idle just to follow the same logic as in the
-     * generic modem implementation. */
-    nm_clear_g_source(&priv->idle_id_ip6);
-    priv->idle_id_ip6 = g_idle_add((GSourceFunc) stage3_ip6_done, self);
-
-    return NM_ACT_STAGE_RETURN_POSTPONE;
+    nm_modem_emit_signal_new_config_success(modem, addr_family, l3cd, do_auto, iid);
 }
 
 /*****************************************************************************/
 /* Disconnect */
 
 typedef struct {
-    NMModemBroadband *         self;
+    NMModemBroadband          *self;
     _NMModemDisconnectCallback callback;
     gpointer                   callback_user_data;
-    GCancellable *             cancellable;
+    GCancellable              *cancellable;
     gboolean                   warn;
 } DisconnectContext;
 
@@ -1230,7 +1219,7 @@ disconnect_context_complete(DisconnectContext *ctx, GError *error)
 static void
 disconnect_context_complete_on_idle(gpointer user_data, GCancellable *cancellable)
 {
-    DisconnectContext *ctx                = user_data;
+    DisconnectContext    *ctx             = user_data;
     gs_free_error GError *cancelled_error = NULL;
 
     g_cancellable_set_error_if_cancelled(cancellable, &cancelled_error);
@@ -1240,9 +1229,9 @@ disconnect_context_complete_on_idle(gpointer user_data, GCancellable *cancellabl
 static void
 simple_disconnect_ready(GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
-    MMModemSimple *    modem_iface = MM_MODEM_SIMPLE(source_object);
-    DisconnectContext *ctx         = user_data;
-    gs_free_error GError *error    = NULL;
+    MMModemSimple        *modem_iface = MM_MODEM_SIMPLE(source_object);
+    DisconnectContext    *ctx         = user_data;
+    gs_free_error GError *error       = NULL;
 
     if (!mm_modem_simple_disconnect_finish(modem_iface, res, &error)) {
         if (ctx->warn && !g_error_matches(error, G_DBUS_ERROR, G_DBUS_ERROR_SERVICE_UNKNOWN)) {
@@ -1256,13 +1245,13 @@ simple_disconnect_ready(GObject *source_object, GAsyncResult *res, gpointer user
 }
 
 static void
-disconnect(NMModem *                  modem,
+disconnect(NMModem                   *modem,
            gboolean                   warn,
-           GCancellable *             cancellable,
+           GCancellable              *cancellable,
            _NMModemDisconnectCallback callback,
            gpointer                   user_data)
 {
-    NMModemBroadband * self = NM_MODEM_BROADBAND(modem);
+    NMModemBroadband  *self = NM_MODEM_BROADBAND(modem);
     DisconnectContext *ctx;
 
     connect_context_clear(self);
@@ -1339,11 +1328,11 @@ mm_state_to_nm(MMModemState mm_state)
 }
 
 static void
-modem_state_changed(MMModem *                modem,
+modem_state_changed(MMModem                 *modem,
                     MMModemState             old_state,
                     MMModemState             new_state,
                     MMModemStateChangeReason reason,
-                    NMModemBroadband *       self)
+                    NMModemBroadband        *self)
 {
     /* After the SIM is unlocked MM1 will move the device to INITIALIZING which
      * is an unavailable state.  That makes state handling confusing here, so
@@ -1381,7 +1370,7 @@ static void
 get_sim_ready(MMModem *modem, GAsyncResult *res, NMModemBroadband *self)
 {
     GError *error = NULL;
-    MMSim * new_sim;
+    MMSim  *new_sim;
 
     new_sim = mm_modem_get_sim_finish(modem, res, &error);
     if (new_sim != self->_priv.sim_iface) {
@@ -1515,12 +1504,12 @@ nm_modem_broadband_init(NMModemBroadband *self)
 NMModem *
 nm_modem_broadband_new(GObject *object, GError **error)
 {
-    MMObject *         modem_object;
-    MMModem *          modem_iface;
-    MMModem3gpp *      modem_3gpp_iface;
+    MMObject          *modem_object;
+    MMModem           *modem_iface;
+    MMModem3gpp       *modem_3gpp_iface;
     const char *const *drivers;
-    const char *       operator_code = NULL;
-    gs_free char *     driver        = NULL;
+    const char        *operator_code = NULL;
+    gs_free char      *driver        = NULL;
 
     g_return_val_if_fail(MM_IS_OBJECT(object), NULL);
     modem_object = MM_OBJECT(object);
@@ -1564,30 +1553,27 @@ nm_modem_broadband_new(GObject *object, GError **error)
 static void
 dispose(GObject *object)
 {
-    NMModemBroadband *       self = NM_MODEM_BROADBAND(object);
+    NMModemBroadband        *self = NM_MODEM_BROADBAND(object);
     NMModemBroadbandPrivate *priv = NM_MODEM_BROADBAND_GET_PRIVATE(self);
 
-    nm_clear_g_source(&priv->idle_id_ip4);
-    nm_clear_g_source(&priv->idle_id_ip6);
-
     connect_context_clear(self);
-    g_clear_object(&self->_priv.ipv4_config);
-    g_clear_object(&self->_priv.ipv6_config);
-    g_clear_object(&self->_priv.bearer);
+    g_clear_object(&priv->ipv4_config);
+    g_clear_object(&priv->ipv6_config);
+    g_clear_object(&priv->bearer);
 
-    if (self->_priv.modem_iface) {
-        g_signal_handlers_disconnect_by_data(self->_priv.modem_iface, self);
-        g_clear_object(&self->_priv.modem_iface);
+    if (priv->modem_iface) {
+        g_signal_handlers_disconnect_by_data(priv->modem_iface, self);
+        g_clear_object(&priv->modem_iface);
     }
 
-    if (self->_priv.modem_3gpp_iface) {
-        g_signal_handlers_disconnect_by_data(self->_priv.modem_3gpp_iface, self);
-        g_clear_object(&self->_priv.modem_3gpp_iface);
+    if (priv->modem_3gpp_iface) {
+        g_signal_handlers_disconnect_by_data(priv->modem_3gpp_iface, self);
+        g_clear_object(&priv->modem_3gpp_iface);
     }
 
-    g_clear_object(&self->_priv.simple_iface);
-    g_clear_object(&self->_priv.sim_iface);
-    g_clear_object(&self->_priv.modem_object);
+    g_clear_object(&priv->simple_iface);
+    g_clear_object(&priv->sim_iface);
+    g_clear_object(&priv->modem_object);
 
     G_OBJECT_CLASS(nm_modem_broadband_parent_class)->dispose(object);
 }
@@ -1603,8 +1589,7 @@ nm_modem_broadband_class_init(NMModemBroadbandClass *klass)
     object_class->set_property = set_property;
 
     modem_class->get_capabilities                       = get_capabilities;
-    modem_class->static_stage3_ip4_config_start         = static_stage3_ip4_config_start;
-    modem_class->stage3_ip6_config_request              = stage3_ip6_config_request;
+    modem_class->stage3_ip_config_start                 = stage3_ip_config_start;
     modem_class->disconnect                             = disconnect;
     modem_class->deactivate_cleanup                     = deactivate_cleanup;
     modem_class->set_mm_enabled                         = set_mm_enabled;
