@@ -7,11 +7,11 @@
 
 #include "nm-modem-ofono.h"
 
-#include "nm-core-internal.h"
+#include "libnm-core-intern/nm-core-internal.h"
 #include "devices/nm-device-private.h"
 #include "nm-modem.h"
-#include "platform/nm-platform.h"
-#include "nm-ip4-config.h"
+#include "libnm-platform/nm-platform.h"
+#include "nm-l3-config-data.h"
 
 #define VARIANT_IS_OF_TYPE_BOOLEAN(v) \
     ((v) != NULL && (g_variant_is_of_type((v), G_VARIANT_TYPE_BOOLEAN)))
@@ -47,7 +47,7 @@ typedef struct {
     gboolean modem_online;
     gboolean gprs_attached;
 
-    NMIP4Config *ip4_config;
+    NML3ConfigData *l3cd_4;
 } NMModemOfonoPrivate;
 
 struct _NMModemOfono {
@@ -75,7 +75,7 @@ G_DEFINE_TYPE(NMModemOfono, nm_modem_ofono, NM_TYPE_MODEM)
         if (nm_logging_enabled(_level, (_NMLOG_DOMAIN))) {                  \
             NMModemOfono *const __self = (self);                            \
             char                __prefix_name[128];                         \
-            const char *        __uid;                                      \
+            const char         *__uid;                                      \
                                                                             \
             _nm_log(_level,                                                 \
                     (_NMLOG_DOMAIN),                                        \
@@ -97,7 +97,7 @@ G_DEFINE_TYPE(NMModemOfono, nm_modem_ofono, NM_TYPE_MODEM)
 /*****************************************************************************/
 
 static void
-get_capabilities(NMModem *                  _self,
+get_capabilities(NMModem                   *_self,
                  NMDeviceModemCapabilities *modem_caps,
                  NMDeviceModemCapabilities *current_caps)
 {
@@ -112,7 +112,7 @@ update_modem_state(NMModemOfono *self)
     NMModemOfonoPrivate *priv      = NM_MODEM_OFONO_GET_PRIVATE(self);
     NMModemState         state     = nm_modem_get_state(NM_MODEM(self));
     NMModemState         new_state = NM_MODEM_STATE_DISABLED;
-    const char *         reason    = NULL;
+    const char          *reason    = NULL;
 
     _LOGI("'Attached': %s 'Online': %s 'IMSI': %s",
           priv->gprs_attached ? "true" : "false",
@@ -137,10 +137,10 @@ update_modem_state(NMModemOfono *self)
 
 /* Disconnect */
 typedef struct {
-    NMModemOfono *             self;
+    NMModemOfono              *self;
     _NMModemDisconnectCallback callback;
     gpointer                   callback_user_data;
-    GCancellable *             cancellable;
+    GCancellable              *cancellable;
     gboolean                   warn;
 } DisconnectContext;
 
@@ -157,7 +157,7 @@ disconnect_context_complete(DisconnectContext *ctx, GError *error)
 static void
 disconnect_context_complete_on_idle(gpointer user_data, GCancellable *cancellable)
 {
-    DisconnectContext *ctx      = user_data;
+    DisconnectContext    *ctx   = user_data;
     gs_free_error GError *error = NULL;
 
     if (!g_cancellable_set_error_if_cancelled(cancellable, &error)) {
@@ -172,10 +172,10 @@ disconnect_context_complete_on_idle(gpointer user_data, GCancellable *cancellabl
 static void
 disconnect_done(GObject *source, GAsyncResult *result, gpointer user_data)
 {
-    DisconnectContext *ctx       = user_data;
-    NMModemOfono *     self      = ctx->self;
-    gs_free_error GError *error  = NULL;
-    gs_unref_variant GVariant *v = NULL;
+    DisconnectContext         *ctx   = user_data;
+    NMModemOfono              *self  = ctx->self;
+    gs_free_error GError      *error = NULL;
+    gs_unref_variant GVariant *v     = NULL;
 
     v = g_dbus_proxy_call_finish(G_DBUS_PROXY(source), result, &error);
     if (g_error_matches(error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
@@ -193,15 +193,15 @@ disconnect_done(GObject *source, GAsyncResult *result, gpointer user_data)
 }
 
 static void
-disconnect(NMModem *                  modem,
+disconnect(NMModem                   *modem,
            gboolean                   warn,
-           GCancellable *             cancellable,
+           GCancellable              *cancellable,
            _NMModemDisconnectCallback callback,
            gpointer                   user_data)
 {
-    NMModemOfono *       self = NM_MODEM_OFONO(modem);
+    NMModemOfono        *self = NM_MODEM_OFONO(modem);
     NMModemOfonoPrivate *priv = NM_MODEM_OFONO_GET_PRIVATE(self);
-    DisconnectContext *  ctx;
+    DisconnectContext   *ctx;
     NMModemState         state = nm_modem_get_state(NM_MODEM(self));
 
     _LOGD("warn: %s modem_state: %s", warn ? "TRUE" : "FALSE", nm_modem_state_to_string(state));
@@ -235,12 +235,12 @@ disconnect(NMModem *                  modem,
 static void
 deactivate_cleanup(NMModem *modem, NMDevice *device, gboolean stop_ppp_manager)
 {
-    NMModemOfono *       self = NM_MODEM_OFONO(modem);
+    NMModemOfono        *self = NM_MODEM_OFONO(modem);
     NMModemOfonoPrivate *priv = NM_MODEM_OFONO_GET_PRIVATE(self);
 
     /* TODO: cancel SimpleConnect() if any */
 
-    g_clear_object(&priv->ip4_config);
+    nm_clear_l3cd(&priv->l3cd_4);
 
     NM_MODEM_CLASS(nm_modem_ofono_parent_class)
         ->deactivate_cleanup(modem, device, stop_ppp_manager);
@@ -249,9 +249,9 @@ deactivate_cleanup(NMModem *modem, NMDevice *device, gboolean stop_ppp_manager)
 static gboolean
 check_connection_compatible_with_modem(NMModem *modem, NMConnection *connection, GError **error)
 {
-    NMModemOfono *       self = NM_MODEM_OFONO(modem);
+    NMModemOfono        *self = NM_MODEM_OFONO(modem);
     NMModemOfonoPrivate *priv = NM_MODEM_OFONO_GET_PRIVATE(self);
-    const char *         id;
+    const char          *id;
 
     if (!_nm_connection_check_main_setting(connection, NM_SETTING_GSM_SETTING_NAME, NULL)) {
         nm_utils_error_set(error,
@@ -290,7 +290,7 @@ check_connection_compatible_with_modem(NMModem *modem, NMConnection *connection,
 static void
 handle_sim_property(GDBusProxy *proxy, const char *property, GVariant *v, gpointer user_data)
 {
-    NMModemOfono *       self = NM_MODEM_OFONO(user_data);
+    NMModemOfono        *self = NM_MODEM_OFONO(user_data);
     NMModemOfonoPrivate *priv = NM_MODEM_OFONO_GET_PRIVATE(self);
 
     if (g_strcmp0(property, "SubscriberIdentity") == 0 && VARIANT_IS_OF_TYPE_STRING(v)) {
@@ -316,23 +316,22 @@ handle_sim_property(GDBusProxy *proxy, const char *property, GVariant *v, gpoint
 static void
 sim_property_changed(GDBusProxy *proxy, const char *property, GVariant *v, gpointer user_data)
 {
-    GVariant *v_child = g_variant_get_child_value(v, 0);
+    gs_unref_variant GVariant *v_child = g_variant_get_child_value(v, 0);
 
     handle_sim_property(proxy, property, v_child, user_data);
-    g_variant_unref(v_child);
 }
 
 static void
 sim_get_properties_done(GObject *source, GAsyncResult *result, gpointer user_data)
 {
-    NMModemOfono *       self;
-    NMModemOfonoPrivate *priv;
-    gs_free_error GError *error             = NULL;
+    NMModemOfono              *self;
+    NMModemOfonoPrivate       *priv;
+    gs_free_error GError      *error        = NULL;
     gs_unref_variant GVariant *v_properties = NULL;
     gs_unref_variant GVariant *v_dict       = NULL;
-    GVariant *                 v;
+    gs_unref_variant GVariant *v            = NULL;
     GVariantIter               i;
-    const char *               property;
+    const char                *property;
 
     v_properties =
         _nm_dbus_proxy_call_finish(G_DBUS_PROXY(source), result, G_VARIANT_TYPE("(a{sv})"), &error);
@@ -370,19 +369,18 @@ sim_get_properties_done(GObject *source, GAsyncResult *result, gpointer user_dat
      */
 
     g_variant_iter_init(&i, v_dict);
-    while (g_variant_iter_next(&i, "{&sv}", &property, &v)) {
+    while (g_variant_iter_loop(&i, "{&sv}", &property, &v)) {
         handle_sim_property(NULL, property, v, self);
-        g_variant_unref(v);
     }
 }
 
 static void
 _sim_proxy_new_cb(GObject *source, GAsyncResult *result, gpointer user_data)
 {
-    NMModemOfono *       self;
-    NMModemOfonoPrivate *priv;
+    NMModemOfono         *self;
+    NMModemOfonoPrivate  *priv;
     gs_free_error GError *error = NULL;
-    GDBusProxy *          proxy;
+    GDBusProxy           *proxy;
 
     proxy = g_dbus_proxy_new_for_bus_finish(result, &error);
     if (!proxy && g_error_matches(error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
@@ -453,7 +451,7 @@ handle_sim_iface(NMModemOfono *self, gboolean found)
 static void
 handle_connman_property(GDBusProxy *proxy, const char *property, GVariant *v, gpointer user_data)
 {
-    NMModemOfono *       self = NM_MODEM_OFONO(user_data);
+    NMModemOfono        *self = NM_MODEM_OFONO(user_data);
     NMModemOfonoPrivate *priv = NM_MODEM_OFONO_GET_PRIVATE(self);
 
     if (g_strcmp0(property, "Attached") == 0 && VARIANT_IS_OF_TYPE_BOOLEAN(v)) {
@@ -477,23 +475,22 @@ handle_connman_property(GDBusProxy *proxy, const char *property, GVariant *v, gp
 static void
 connman_property_changed(GDBusProxy *proxy, const char *property, GVariant *v, gpointer user_data)
 {
-    GVariant *v_child = g_variant_get_child_value(v, 0);
+    gs_unref_variant GVariant *v_child = g_variant_get_child_value(v, 0);
 
     handle_connman_property(proxy, property, v_child, user_data);
-    g_variant_unref(v_child);
 }
 
 static void
 connman_get_properties_done(GObject *source, GAsyncResult *result, gpointer user_data)
 {
-    NMModemOfono *       self;
-    NMModemOfonoPrivate *priv;
-    gs_free_error GError *error             = NULL;
+    NMModemOfono              *self;
+    NMModemOfonoPrivate       *priv;
+    gs_free_error GError      *error        = NULL;
     gs_unref_variant GVariant *v_properties = NULL;
     gs_unref_variant GVariant *v_dict       = NULL;
-    GVariant *                 v;
+    gs_unref_variant GVariant *v            = NULL;
     GVariantIter               i;
-    const char *               property;
+    const char                *property;
 
     v_properties =
         _nm_dbus_proxy_call_finish(G_DBUS_PROXY(source), result, G_VARIANT_TYPE("(a{sv})"), &error);
@@ -523,19 +520,18 @@ connman_get_properties_done(GObject *source, GAsyncResult *result, gpointer user
      */
 
     g_variant_iter_init(&i, v_dict);
-    while (g_variant_iter_next(&i, "{&sv}", &property, &v)) {
+    while (g_variant_iter_loop(&i, "{&sv}", &property, &v)) {
         handle_connman_property(NULL, property, v, self);
-        g_variant_unref(v);
     }
 }
 
 static void
 _connman_proxy_new_cb(GObject *source, GAsyncResult *result, gpointer user_data)
 {
-    NMModemOfono *       self;
-    NMModemOfonoPrivate *priv;
+    NMModemOfono         *self;
+    NMModemOfonoPrivate  *priv;
     gs_free_error GError *error = NULL;
-    GDBusProxy *          proxy;
+    GDBusProxy           *proxy;
 
     proxy = g_dbus_proxy_new_for_bus_finish(result, &error);
     if (!proxy && g_error_matches(error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
@@ -610,7 +606,7 @@ handle_connman_iface(NMModemOfono *self, gboolean found)
 static void
 handle_modem_property(GDBusProxy *proxy, const char *property, GVariant *v, gpointer user_data)
 {
-    NMModemOfono *       self = NM_MODEM_OFONO(user_data);
+    NMModemOfono        *self = NM_MODEM_OFONO(user_data);
     NMModemOfonoPrivate *priv = NM_MODEM_OFONO_GET_PRIVATE(self);
 
     if ((g_strcmp0(property, "Online") == 0) && VARIANT_IS_OF_TYPE_BOOLEAN(v)) {
@@ -659,14 +655,14 @@ modem_property_changed(GDBusProxy *proxy, const char *property, GVariant *v, gpo
 static void
 modem_get_properties_done(GObject *source, GAsyncResult *result, gpointer user_data)
 {
-    NMModemOfono *       self;
-    NMModemOfonoPrivate *priv;
-    gs_free_error GError *error             = NULL;
+    NMModemOfono              *self;
+    NMModemOfonoPrivate       *priv;
+    gs_free_error GError      *error        = NULL;
     gs_unref_variant GVariant *v_properties = NULL;
     gs_unref_variant GVariant *v_dict       = NULL;
-    GVariant *                 v;
+    GVariant                  *v;
     GVariantIter               i;
-    const char *               property;
+    const char                *property;
 
     v_properties =
         _nm_dbus_proxy_call_finish(G_DBUS_PROXY(source), result, G_VARIANT_TYPE("(a{sv})"), &error);
@@ -700,19 +696,18 @@ modem_get_properties_done(GObject *source, GAsyncResult *result, gpointer user_d
      */
 
     g_variant_iter_init(&i, v_dict);
-    while (g_variant_iter_next(&i, "{&sv}", &property, &v)) {
+    while (g_variant_iter_loop(&i, "{&sv}", &property, &v)) {
         handle_modem_property(NULL, property, v, self);
-        g_variant_unref(v);
     }
 }
 
 static void
 stage1_prepare_done(GObject *source, GAsyncResult *result, gpointer user_data)
 {
-    NMModemOfono *       self;
-    NMModemOfonoPrivate *priv;
-    gs_free_error GError *error  = NULL;
-    gs_unref_variant GVariant *v = NULL;
+    NMModemOfono              *self;
+    NMModemOfonoPrivate       *priv;
+    gs_free_error GError      *error = NULL;
+    gs_unref_variant GVariant *v     = NULL;
 
     v = g_dbus_proxy_call_finish(G_DBUS_PROXY(source), result, &error);
     if (g_error_matches(error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
@@ -726,35 +721,30 @@ stage1_prepare_done(GObject *source, GAsyncResult *result, gpointer user_data)
     nm_clear_pointer(&priv->connect_properties, g_hash_table_destroy);
 
     if (error) {
-        _LOGW("connection failed: %s", error->message);
-
-        nm_modem_emit_prepare_result(NM_MODEM(self), FALSE, NM_DEVICE_STATE_REASON_MODEM_BUSY);
-        /*
-         * FIXME: add code to check for InProgress so that the
-         * connection doesn't continue to try and activate,
-         * leading to the connection being disabled, and a 5m
-         * timeout...
-         */
+        if (!g_strstr_len(error->message,
+                          NM_STRLEN(OFONO_ERROR_IN_PROGRESS),
+                          OFONO_ERROR_IN_PROGRESS)) {
+            nm_modem_emit_prepare_result(NM_MODEM(self), FALSE, NM_DEVICE_STATE_REASON_MODEM_BUSY);
+        }
     }
 }
 
 static void
-context_property_changed(GDBusProxy *proxy, const char *property, GVariant *v, gpointer user_data)
+handle_settings(GVariant *v_dict, gpointer user_data)
 {
-    NMModemOfono *       self = NM_MODEM_OFONO(user_data);
+    NMModemOfono        *self = NM_MODEM_OFONO(user_data);
     NMModemOfonoPrivate *priv = NM_MODEM_OFONO_GET_PRIVATE(self);
-    NMPlatformIP4Address addr;
-    gboolean             ret          = FALSE;
-    gs_unref_variant GVariant *v_dict = NULL;
-    const char *               interface;
-    const char *               s;
-    const char **              array, **iter;
-    guint32                    address_network, gateway_network;
-    guint32                    ip4_route_table, ip4_route_metric;
-    int                        ifindex;
-    GError *                   error = NULL;
+    char                 sbuf[sizeof(_nm_utils_to_string_buffer)];
+    NMPlatformIP4Address address;
+    gboolean             ret = FALSE;
+    const char          *interface;
+    const char          *s;
+    const char         **array;
+    guint32              address_network, gateway_network;
+    int                  ifindex;
+    GError              *error = NULL;
 
-    _LOGD("PropertyChanged: %s", property);
+    //_LOGD("PropertyChanged: %s", property);
 
     /*
      * TODO: might be a good idea and re-factor this to mimic bluez-device,
@@ -762,15 +752,6 @@ context_property_changed(GDBusProxy *proxy, const char *property, GVariant *v, g
      * handle the action.
      */
 
-    if (g_strcmp0(property, "Settings") != 0)
-        return;
-
-    v_dict = g_variant_get_child_value(v, 0);
-    if (!v_dict) {
-        _LOGW("error getting IPv4 Settings: no v_dict");
-        goto out;
-    }
-
     _LOGI("IPv4 static Settings:");
 
     if (!g_variant_lookup(v_dict, "Interface", "&s", &interface)) {
@@ -792,12 +773,14 @@ context_property_changed(GDBusProxy *proxy, const char *property, GVariant *v, g
     }
 
     ifindex = nm_modem_get_ip_ifindex(NM_MODEM(self));
-    nm_assert(ifindex > 0);
+    g_return_if_fail(ifindex > 0);
 
-    /* TODO: verify handling of ip4_config; check other places it's used... */
-    g_clear_object(&priv->ip4_config);
+    /* TODO: verify handling of l3cd_4; check other places it's used... */
+    nm_clear_l3cd(&priv->l3cd_4);
 
-    priv->ip4_config = nm_ip4_config_new(nm_platform_get_multi_idx(NM_PLATFORM_GET), ifindex);
+    priv->l3cd_4 = nm_l3_config_data_new(nm_platform_get_multi_idx(NM_PLATFORM_GET),
+                                         ifindex,
+                                         NM_IP_CONFIG_SOURCE_WWAN);
 
     if (!g_variant_lookup(v_dict, "Address", "&s", &s)) {
         _LOGW("Settings 'Address' missing");
@@ -807,10 +790,12 @@ context_property_changed(GDBusProxy *proxy, const char *property, GVariant *v, g
         _LOGW("can't convert 'Address' %s to addr", s ?: "");
         goto out;
     }
-    memset(&addr, 0, sizeof(addr));
-    addr.ifindex     = ifindex;
-    addr.address     = address_network;
-    addr.addr_source = NM_IP_CONFIG_SOURCE_WWAN;
+
+    address = (NMPlatformIP4Address){
+        .ifindex     = ifindex,
+        .address     = address_network,
+        .addr_source = NM_IP_CONFIG_SOURCE_WWAN,
+    };
 
     if (!g_variant_lookup(v_dict, "Netmask", "&s", &s)) {
         _LOGW("Settings 'Netmask' missing");
@@ -820,10 +805,10 @@ context_property_changed(GDBusProxy *proxy, const char *property, GVariant *v, g
         _LOGW("invalid 'Netmask': %s", s ?: "");
         goto out;
     }
-    addr.plen = nm_utils_ip4_netmask_to_prefix(address_network);
+    address.plen = nm_utils_ip4_netmask_to_prefix(address_network);
 
-    _LOGI("Address: %s", nm_platform_ip4_address_to_string(&addr, NULL, 0));
-    nm_ip4_config_add_address(priv->ip4_config, &addr);
+    _LOGI("Address: %s", nm_platform_ip4_address_to_string(&address, sbuf, sizeof(sbuf)));
+    nm_l3_config_data_add_address_4(priv->l3cd_4, &address);
 
     if (!g_variant_lookup(v_dict, "Gateway", "&s", &s) || !s) {
         _LOGW("Settings 'Gateway' missing");
@@ -833,17 +818,18 @@ context_property_changed(GDBusProxy *proxy, const char *property, GVariant *v, g
         _LOGW("invalid 'Gateway': %s", s);
         goto out;
     }
-    nm_modem_get_route_parameters(NM_MODEM(self), &ip4_route_table, &ip4_route_metric, NULL, NULL);
     {
         const NMPlatformIP4Route r = {
             .rt_source     = NM_IP_CONFIG_SOURCE_WWAN,
             .gateway       = gateway_network,
-            .table_coerced = nm_platform_route_table_coerce(ip4_route_table),
-            .metric        = ip4_route_metric,
+            .table_any     = TRUE,
+            .table_coerced = 0,
+            .metric_any    = TRUE,
+            .metric        = 0,
         };
 
         _LOGI("Gateway: %s", s);
-        nm_ip4_config_add_route(priv->ip4_config, &r, NULL);
+        nm_l3_config_data_add_route_4(priv->l3cd_4, &r);
     }
 
     if (!g_variant_lookup(v_dict, "DomainNameServers", "^a&s", &array)) {
@@ -851,52 +837,48 @@ context_property_changed(GDBusProxy *proxy, const char *property, GVariant *v, g
         goto out;
     }
     if (array) {
-        for (iter = array; *iter; iter++) {
-            if (nm_utils_parse_inaddr_bin(AF_INET, *iter, NULL, &address_network)
-                && address_network) {
-                _LOGI("DNS: %s", *iter);
-                nm_ip4_config_add_nameserver(priv->ip4_config, address_network);
-            } else {
-                _LOGW("invalid NameServer: %s", *iter);
-            }
-        }
+        gboolean any_good = FALSE;
 
-        if (iter == array) {
+        for (; array[0]; array++) {
+            if (!nm_utils_parse_inaddr_bin(AF_INET, *array, NULL, &address_network)
+                || !address_network) {
+                _LOGW("invalid NameServer: %s", *array);
+                continue;
+            }
+            any_good = TRUE;
+            _LOGI("DNS: %s", *array);
+            nm_l3_config_data_add_nameserver(priv->l3cd_4, AF_INET, &address_network);
+        }
+        if (!any_good) {
             _LOGW("Settings: 'DomainNameServers': none specified");
-            g_free(array);
             goto out;
         }
-        g_free(array);
     }
 
     if (g_variant_lookup(v_dict, "MessageProxy", "&s", &s)) {
         _LOGI("MessageProxy: %s", s);
         if (s && nm_utils_parse_inaddr_bin(AF_INET, s, NULL, &address_network)) {
-            nm_modem_get_route_parameters(NM_MODEM(self),
-                                          &ip4_route_table,
-                                          &ip4_route_metric,
-                                          NULL,
-                                          NULL);
+            const NMPlatformIP4Route mms_route = {
+                .network       = address_network,
+                .plen          = 32,
+                .gateway       = gateway_network,
+                .table_any     = TRUE,
+                .table_coerced = 0,
+                .metric_any    = TRUE,
+                .metric        = 0,
+            };
 
-            {
-                const NMPlatformIP4Route mms_route = {
-                    .network       = address_network,
-                    .plen          = 32,
-                    .gateway       = gateway_network,
-                    .table_coerced = nm_platform_route_table_coerce(ip4_route_table),
-                    .metric        = ip4_route_metric,
-                };
-
-                nm_ip4_config_add_route(priv->ip4_config, &mms_route, NULL);
-            }
-        } else {
+            nm_l3_config_data_add_route_4(priv->l3cd_4, &mms_route);
+        } else
             _LOGW("invalid MessageProxy: %s", s);
-        }
     }
 
     ret = TRUE;
 
 out:
+    if (priv->l3cd_4)
+        nm_l3_config_data_seal(priv->l3cd_4);
+
     if (nm_modem_get_state(NM_MODEM(self)) != NM_MODEM_STATE_CONNECTED) {
         _LOGI("emitting PREPARE_RESULT: %s", ret ? "TRUE" : "FALSE");
         nm_modem_emit_prepare_result(NM_MODEM(self),
@@ -909,39 +891,132 @@ out:
     }
 }
 
-static NMActStageReturn
-static_stage3_ip4_config_start(NMModem *            modem,
-                               NMActRequest *       req,
-                               NMDeviceStateReason *out_failure_reason)
+static void
+context_property_changed(GDBusProxy *proxy, const char *property, GVariant *v, gpointer user_data)
 {
-    NMModemOfono *       self  = NM_MODEM_OFONO(modem);
-    NMModemOfonoPrivate *priv  = NM_MODEM_OFONO_GET_PRIVATE(self);
-    GError *             error = NULL;
+    NMModemOfono              *self   = NM_MODEM_OFONO(user_data);
+    gs_unref_variant GVariant *v_dict = NULL;
 
-    if (!priv->ip4_config) {
-        _LOGD("IP4 config not ready(?)");
-        return NM_ACT_STAGE_RETURN_FAILURE;
+    _LOGD("PropertyChanged: %s", property);
+
+    if (g_strcmp0(property, "Settings") != 0)
+        return;
+
+    v_dict = g_variant_get_child_value(v, 0);
+    if (!v_dict) {
+        _LOGW("ofono: (%s): error getting IPv4 Settings", nm_modem_get_uid(NM_MODEM(self)));
+        return;
     }
 
+    g_assert(g_variant_is_of_type(v_dict, G_VARIANT_TYPE_VARDICT));
+
+    handle_settings(v_dict, user_data);
+}
+
+static void
+stage3_ip_config_start(NMModem *modem, int addr_family, NMModemIPMethod ip_method)
+{
+    NMModemOfono         *self  = NM_MODEM_OFONO(modem);
+    NMModemOfonoPrivate  *priv  = NM_MODEM_OFONO_GET_PRIVATE(self);
+    gs_free_error GError *error = NULL;
+
     _LOGD("IP4 config is done; setting modem_state -> CONNECTED");
-    g_signal_emit_by_name(self, NM_MODEM_IP4_CONFIG_RESULT, priv->ip4_config, error);
 
-    /* Signal listener takes ownership of the IP4Config */
-    priv->ip4_config = NULL;
+    if (!NM_IS_IPv4(addr_family) || ip_method == NM_MODEM_IP_METHOD_AUTO) {
+        nm_modem_emit_signal_new_config_success(modem, addr_family, NULL, TRUE, NULL);
+        goto out;
+    }
 
+    if (!priv->l3cd_4) {
+        nm_utils_error_set(&error, NM_UTILS_ERROR_UNKNOWN, "IP config not received");
+        nm_modem_emit_signal_new_config_failure(modem,
+                                                addr_family,
+                                                NM_DEVICE_STATE_REASON_IP_CONFIG_UNAVAILABLE,
+                                                error);
+        goto out;
+    }
+
+    nm_modem_emit_signal_new_config_success(modem, addr_family, priv->l3cd_4, FALSE, NULL);
+
+out:
     nm_modem_set_state(NM_MODEM(self),
                        NM_MODEM_STATE_CONNECTED,
                        nm_modem_state_to_string(NM_MODEM_STATE_CONNECTED));
-    return NM_ACT_STAGE_RETURN_POSTPONE;
+}
+
+static void
+context_properties_cb(GDBusProxy *proxy, GAsyncResult *result, gpointer user_data)
+{
+    NMModemOfono              *self;
+    NMModemOfonoPrivate       *priv;
+    gs_free_error GError      *error      = NULL;
+    gs_unref_variant GVariant *properties = NULL;
+    gs_unref_variant GVariant *settings   = NULL;
+    gs_unref_variant GVariant *v_dict     = NULL;
+    gboolean                   active;
+
+    self = NM_MODEM_OFONO(user_data);
+    priv = NM_MODEM_OFONO_GET_PRIVATE(self);
+
+    properties = g_dbus_proxy_call_finish(proxy, result, &error);
+
+    if (!properties) {
+        _LOGW("ofono: connection failed: no context properties returned %s", error->message);
+        g_clear_error(&error);
+        goto error;
+    }
+
+    v_dict = g_variant_get_child_value(properties, 0);
+    if (!v_dict || !g_variant_is_of_type(v_dict, G_VARIANT_TYPE_VARDICT)) {
+        _LOGW("ofono: connection failed; could not read connection properties");
+        goto error;
+    }
+
+    if (!g_variant_lookup(v_dict, "Active", "b", &active)) {
+        _LOGW("ofono: connection failed; can not read 'Active' property");
+        goto error;
+    }
+
+    /* Watch for custom ofono PropertyChanged signals */
+    _nm_dbus_signal_connect(priv->context_proxy,
+                            "PropertyChanged",
+                            G_VARIANT_TYPE("(sv)"),
+                            G_CALLBACK(context_property_changed),
+                            self);
+
+    if (active) {
+        _LOGD("ofono: connection is already Active");
+
+        settings = g_variant_lookup_value(v_dict, "Settings", G_VARIANT_TYPE_VARDICT);
+        if (settings == NULL) {
+            _LOGW("ofono: connection failed; can not read 'Settings' property");
+            goto error;
+        }
+
+        handle_settings(settings, user_data);
+    } else {
+        g_dbus_proxy_call(priv->context_proxy,
+                          "SetProperty",
+                          g_variant_new("(sv)", "Active", g_variant_new("b", TRUE)),
+                          G_DBUS_CALL_FLAGS_NONE,
+                          20000,
+                          NULL,
+                          (GAsyncReadyCallback) stage1_prepare_done,
+                          self);
+    }
+    return;
+
+error:
+    nm_modem_emit_prepare_result(NM_MODEM(self), FALSE, NM_DEVICE_STATE_REASON_MODEM_BUSY);
 }
 
 static void
 context_proxy_new_cb(GObject *source, GAsyncResult *result, gpointer user_data)
 {
-    NMModemOfono *       self;
-    NMModemOfonoPrivate *priv;
+    NMModemOfono         *self;
+    NMModemOfonoPrivate  *priv;
     gs_free_error GError *error = NULL;
-    GDBusProxy *          proxy;
+    GDBusProxy           *proxy;
 
     proxy = g_dbus_proxy_new_for_bus_finish(result, &error);
     if (!proxy || g_error_matches(error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
@@ -971,21 +1046,17 @@ context_proxy_new_cb(GObject *source, GAsyncResult *result, gpointer user_data)
      * clear it so that we can gate getting the IP config from oFono
      * on whether or not we have already received them
      */
-    g_clear_object(&priv->ip4_config);
-
-    _nm_dbus_signal_connect(priv->context_proxy,
-                            "PropertyChanged",
-                            G_VARIANT_TYPE("(sv)"),
-                            G_CALLBACK(context_property_changed),
-                            self);
+    nm_clear_l3cd(&priv->l3cd_4);
 
+    /* We need to directly query ConnectionContextinteface to get the current
+     * property values */
     g_dbus_proxy_call(priv->context_proxy,
-                      "SetProperty",
-                      g_variant_new("(sv)", "Active", g_variant_new("b", TRUE)),
+                      "GetProperties",
+                      NULL,
                       G_DBUS_CALL_FLAGS_NONE,
                       20000,
-                      priv->context_proxy_cancellable,
-                      stage1_prepare_done,
+                      NULL,
+                      (GAsyncReadyCallback) context_properties_cb,
                       self);
 }
 
@@ -1016,8 +1087,8 @@ static GHashTable *
 create_connect_properties(NMConnection *connection)
 {
     NMSettingGsm *setting;
-    GHashTable *  properties;
-    const char *  str;
+    GHashTable   *properties;
+    const char   *str;
 
     setting    = nm_connection_get_setting_gsm(connection);
     properties = g_hash_table_new(nm_str_hash, g_str_equal);
@@ -1038,14 +1109,14 @@ create_connect_properties(NMConnection *connection)
 }
 
 static NMActStageReturn
-modem_act_stage1_prepare(NMModem *            modem,
-                         NMConnection *       connection,
+modem_act_stage1_prepare(NMModem             *modem,
+                         NMConnection        *connection,
                          NMDeviceStateReason *out_failure_reason)
 {
-    NMModemOfono *       self = NM_MODEM_OFONO(modem);
+    NMModemOfono        *self = NM_MODEM_OFONO(modem);
     NMModemOfonoPrivate *priv = NM_MODEM_OFONO_GET_PRIVATE(self);
-    const char *         context_id;
-    char **              id = NULL;
+    const char          *context_id;
+    char               **id = NULL;
 
     context_id = nm_connection_get_id(connection);
     id         = g_strsplit(context_id, "/", 0);
@@ -1069,6 +1140,7 @@ modem_act_stage1_prepare(NMModem *            modem,
 
     _LOGI("activating context %s", priv->context_path);
 
+    update_modem_state(self);
     if (nm_modem_get_state(modem) == NM_MODEM_STATE_REGISTERED) {
         do_context_activate(self);
     } else {
@@ -1083,10 +1155,10 @@ modem_act_stage1_prepare(NMModem *            modem,
 static void
 modem_proxy_new_cb(GObject *source, GAsyncResult *result, gpointer user_data)
 {
-    NMModemOfono *       self;
-    NMModemOfonoPrivate *priv;
+    NMModemOfono         *self;
+    NMModemOfonoPrivate  *priv;
     gs_free_error GError *error = NULL;
-    GDBusProxy *          proxy;
+    GDBusProxy           *proxy;
 
     proxy = g_dbus_proxy_new_for_bus_finish(result, &error);
     if (!proxy && g_error_matches(error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
@@ -1128,7 +1200,7 @@ nm_modem_ofono_init(NMModemOfono *self)
 static void
 constructed(GObject *object)
 {
-    NMModemOfono *       self = NM_MODEM_OFONO(object);
+    NMModemOfono        *self = NM_MODEM_OFONO(object);
     NMModemOfonoPrivate *priv = NM_MODEM_OFONO_GET_PRIVATE(self);
 
     priv->modem_proxy_cancellable = g_cancellable_new();
@@ -1179,7 +1251,7 @@ nm_modem_ofono_new(const char *path)
 static void
 dispose(GObject *object)
 {
-    NMModemOfono *       self = NM_MODEM_OFONO(object);
+    NMModemOfono        *self = NM_MODEM_OFONO(object);
     NMModemOfonoPrivate *priv = NM_MODEM_OFONO_GET_PRIVATE(self);
 
     nm_clear_g_cancellable(&priv->modem_proxy_cancellable);
@@ -1192,7 +1264,7 @@ dispose(GObject *object)
         priv->connect_properties = NULL;
     }
 
-    g_clear_object(&priv->ip4_config);
+    nm_clear_l3cd(&priv->l3cd_4);
 
     if (priv->modem_proxy) {
         g_signal_handlers_disconnect_by_data(priv->modem_proxy, self);
@@ -1234,6 +1306,6 @@ nm_modem_ofono_class_init(NMModemOfonoClass *klass)
     modem_class->deactivate_cleanup                     = deactivate_cleanup;
     modem_class->check_connection_compatible_with_modem = check_connection_compatible_with_modem;
 
-    modem_class->modem_act_stage1_prepare       = modem_act_stage1_prepare;
-    modem_class->static_stage3_ip4_config_start = static_stage3_ip4_config_start;
+    modem_class->modem_act_stage1_prepare = modem_act_stage1_prepare;
+    modem_class->stage3_ip_config_start   = stage3_ip_config_start;
 }
