@@ -14,21 +14,21 @@
 
 #if WITH_DHCLIENT
 
-    #include <stdlib.h>
-    #include <unistd.h>
-    #include <stdio.h>
-    #include <netinet/in.h>
-    #include <arpa/inet.h>
-    #include <ctype.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <stdio.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <ctype.h>
 
-    #include "nm-glib-aux/nm-dedup-multi.h"
+#include "libnm-glib-aux/nm-dedup-multi.h"
 
-    #include "nm-utils.h"
-    #include "nm-dhcp-dhclient-utils.h"
-    #include "nm-dhcp-manager.h"
-    #include "NetworkManagerUtils.h"
-    #include "nm-dhcp-listener.h"
-    #include "nm-dhcp-client-logging.h"
+#include "nm-utils.h"
+#include "nm-dhcp-dhclient-utils.h"
+#include "nm-dhcp-manager.h"
+#include "NetworkManagerUtils.h"
+#include "nm-dhcp-listener.h"
+#include "nm-dhcp-client-logging.h"
 
 /*****************************************************************************/
 
@@ -41,16 +41,15 @@ _addr_family_to_path_part(int addr_family)
 
 /*****************************************************************************/
 
-    #define NM_TYPE_DHCP_DHCLIENT (nm_dhcp_dhclient_get_type())
-    #define NM_DHCP_DHCLIENT(obj) \
-        (G_TYPE_CHECK_INSTANCE_CAST((obj), NM_TYPE_DHCP_DHCLIENT, NMDhcpDhclient))
-    #define NM_DHCP_DHCLIENT_CLASS(klass) \
-        (G_TYPE_CHECK_CLASS_CAST((klass), NM_TYPE_DHCP_DHCLIENT, NMDhcpDhclientClass))
-    #define NM_IS_DHCP_DHCLIENT(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), NM_TYPE_DHCP_DHCLIENT))
-    #define NM_IS_DHCP_DHCLIENT_CLASS(klass) \
-        (G_TYPE_CHECK_CLASS_TYPE((klass), NM_TYPE_DHCP_DHCLIENT))
-    #define NM_DHCP_DHCLIENT_GET_CLASS(obj) \
-        (G_TYPE_INSTANCE_GET_CLASS((obj), NM_TYPE_DHCP_DHCLIENT, NMDhcpDhclientClass))
+#define NM_TYPE_DHCP_DHCLIENT (nm_dhcp_dhclient_get_type())
+#define NM_DHCP_DHCLIENT(obj) \
+    (G_TYPE_CHECK_INSTANCE_CAST((obj), NM_TYPE_DHCP_DHCLIENT, NMDhcpDhclient))
+#define NM_DHCP_DHCLIENT_CLASS(klass) \
+    (G_TYPE_CHECK_CLASS_CAST((klass), NM_TYPE_DHCP_DHCLIENT, NMDhcpDhclientClass))
+#define NM_IS_DHCP_DHCLIENT(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), NM_TYPE_DHCP_DHCLIENT))
+#define NM_IS_DHCP_DHCLIENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), NM_TYPE_DHCP_DHCLIENT))
+#define NM_DHCP_DHCLIENT_GET_CLASS(obj) \
+    (G_TYPE_INSTANCE_GET_CLASS((obj), NM_TYPE_DHCP_DHCLIENT, NMDhcpDhclientClass))
 
 typedef struct _NMDhcpDhclient      NMDhcpDhclient;
 typedef struct _NMDhcpDhclientClass NMDhcpDhclientClass;
@@ -60,10 +59,10 @@ static GType nm_dhcp_dhclient_get_type(void);
 /*****************************************************************************/
 
 typedef struct {
-    char *          conf_file;
-    const char *    def_leasefile;
-    char *          lease_file;
-    char *          pid_file;
+    char           *conf_file;
+    const char     *def_leasefile;
+    char           *lease_file;
+    char           *pid_file;
     NMDhcpListener *dhcp_listener;
 } NMDhcpDhclientPrivate;
 
@@ -78,8 +77,8 @@ struct _NMDhcpDhclientClass {
 
 G_DEFINE_TYPE(NMDhcpDhclient, nm_dhcp_dhclient, NM_TYPE_DHCP_CLIENT)
 
-    #define NM_DHCP_DHCLIENT_GET_PRIVATE(self) \
-        _NM_GET_PRIVATE(self, NMDhcpDhclient, NM_IS_DHCP_DHCLIENT)
+#define NM_DHCP_DHCLIENT_GET_PRIVATE(self) \
+    _NM_GET_PRIVATE(self, NMDhcpDhclient, NM_IS_DHCP_DHCLIENT)
 
 /*****************************************************************************/
 
@@ -106,7 +105,7 @@ static char *
 get_dhclient_leasefile(int         addr_family,
                        const char *iface,
                        const char *uuid,
-                       char **     out_preferred_path)
+                       char      **out_preferred_path)
 {
     gs_free char *path = NULL;
 
@@ -146,21 +145,21 @@ get_dhclient_leasefile(int         addr_family,
 }
 
 static gboolean
-merge_dhclient_config(NMDhcpDhclient *    self,
+merge_dhclient_config(NMDhcpDhclient     *self,
                       int                 addr_family,
-                      const char *        iface,
-                      const char *        conf_file,
-                      GBytes *            client_id,
-                      const char *        anycast_addr,
-                      const char *        hostname,
+                      const char         *iface,
+                      const char         *conf_file,
+                      GBytes             *client_id,
+                      const char         *anycast_address,
+                      const char         *hostname,
                       guint32             timeout,
                       gboolean            use_fqdn,
                       NMDhcpHostnameFlags hostname_flags,
-                      const char *        mud_url,
-                      const char *const * reject_servers,
-                      const char *        orig_path,
-                      GBytes **           out_new_client_id,
-                      GError **           error)
+                      const char         *mud_url,
+                      const char *const  *reject_servers,
+                      const char         *orig_path,
+                      GBytes            **out_new_client_id,
+                      GError            **error)
 {
     gs_free char *orig = NULL;
     gs_free char *new  = NULL;
@@ -180,7 +179,7 @@ merge_dhclient_config(NMDhcpDhclient *    self,
     new = nm_dhcp_dhclient_create_config(iface,
                                          addr_family,
                                          client_id,
-                                         anycast_addr,
+                                         anycast_address,
                                          hostname,
                                          timeout,
                                          use_fqdn,
@@ -275,19 +274,19 @@ find_existing_config(NMDhcpDhclient *self, int addr_family, const char *iface, c
  * config file along with the NM options.
  */
 static char *
-create_dhclient_config(NMDhcpDhclient *    self,
+create_dhclient_config(NMDhcpDhclient     *self,
                        int                 addr_family,
-                       const char *        iface,
-                       const char *        uuid,
-                       GBytes *            client_id,
-                       const char *        dhcp_anycast_addr,
-                       const char *        hostname,
+                       const char         *iface,
+                       const char         *uuid,
+                       GBytes             *client_id,
+                       const char         *anycast_address,
+                       const char         *hostname,
                        guint32             timeout,
                        gboolean            use_fqdn,
                        NMDhcpHostnameFlags hostname_flags,
-                       const char *        mud_url,
-                       const char *const * reject_servers,
-                       GBytes **           out_new_client_id)
+                       const char         *mud_url,
+                       const char *const  *reject_servers,
+                       GBytes            **out_new_client_id)
 {
     gs_free char *orig = NULL;
     char *new          = NULL;
@@ -312,7 +311,7 @@ create_dhclient_config(NMDhcpDhclient *    self,
                                iface,
                                new,
                                client_id,
-                               dhcp_anycast_addr,
+                               anycast_address,
                                hostname,
                                timeout,
                                use_fqdn,
@@ -331,29 +330,31 @@ create_dhclient_config(NMDhcpDhclient *    self,
 
 static gboolean
 dhclient_start(NMDhcpClient *client,
-               const char *  mode_opt,
+               const char   *mode_opt,
                gboolean      release,
-               pid_t *       out_pid,
-               int           prefixes,
-               GError **     error)
+               pid_t        *out_pid,
+               GError      **error)
 {
-    NMDhcpDhclient *       self       = NM_DHCP_DHCLIENT(client);
-    NMDhcpDhclientPrivate *priv       = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
+    NMDhcpDhclient              *self = NM_DHCP_DHCLIENT(client);
+    NMDhcpDhclientPrivate       *priv = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
     gs_unref_ptrarray GPtrArray *argv = NULL;
     pid_t                        pid;
-    gs_free_error GError *local = NULL;
-    const char *          iface;
-    const char *          uuid;
-    const char *          system_bus_address;
-    const char *          dhclient_path;
-    char *                binary_name;
-    gs_free char *        cmd_str                  = NULL;
-    gs_free char *        pid_file                 = NULL;
-    gs_free char *        system_bus_address_env   = NULL;
-    gs_free char *        preferred_leasefile_path = NULL;
-    const int             addr_family              = nm_dhcp_client_get_addr_family(client);
+    gs_free_error GError        *local = NULL;
+    const char                  *iface;
+    const char                  *uuid;
+    const char                  *system_bus_address;
+    const char                  *dhclient_path;
+    char                        *binary_name;
+    gs_free char                *cmd_str                  = NULL;
+    gs_free char                *pid_file                 = NULL;
+    gs_free char                *system_bus_address_env   = NULL;
+    gs_free char                *preferred_leasefile_path = NULL;
+    int                          addr_family;
+    const NMDhcpClientConfig    *client_config;
 
     g_return_val_if_fail(!priv->pid_file, FALSE);
+    client_config = nm_dhcp_client_get_config(client);
+    addr_family   = client_config->addr_family;
 
     NM_SET_OUT(out_pid, 0);
 
@@ -363,8 +364,8 @@ dhclient_start(NMDhcpClient *client,
         return FALSE;
     }
 
-    iface = nm_dhcp_client_get_iface(client);
-    uuid  = nm_dhcp_client_get_uuid(client);
+    iface = client_config->iface;
+    uuid  = client_config->uuid;
 
     pid_file = g_strdup_printf(NMRUNDIR "/dhclient%s-%s.pid",
                                _addr_family_to_path_part(addr_family),
@@ -410,9 +411,7 @@ dhclient_start(NMDhcpClient *client,
 
     /* Save the DUID to the leasefile dhclient will actually use */
     if (addr_family == AF_INET6) {
-        if (!nm_dhcp_dhclient_save_duid(priv->lease_file,
-                                        nm_dhcp_client_get_client_id(client),
-                                        &local)) {
+        if (!nm_dhcp_dhclient_save_duid(priv->lease_file, client_config->client_id, &local)) {
             nm_utils_error_set(error,
                                NM_UTILS_ERROR_UNKNOWN,
                                "failed to save DUID to '%s': %s",
@@ -435,7 +434,13 @@ dhclient_start(NMDhcpClient *client,
     if (release)
         g_ptr_array_add(argv, (gpointer) "-r");
 
+    if (!release && client_config->addr_family == AF_INET && client_config->v4.request_broadcast) {
+        g_ptr_array_add(argv, (gpointer) "-B");
+    }
+
     if (addr_family == AF_INET6) {
+        guint prefixes = client_config->v6.needed_prefixes;
+
         g_ptr_array_add(argv, (gpointer) "-6");
 
         if (prefixes > 0 && nm_streq0(mode_opt, "-S")) {
@@ -508,30 +513,27 @@ dhclient_start(NMDhcpClient *client,
 }
 
 static gboolean
-ip4_start(NMDhcpClient *client,
-          const char *  dhcp_anycast_addr,
-          const char *  last_ip4_address,
-          GError **     error)
+ip4_start(NMDhcpClient *client, GError **error)
 {
-    NMDhcpDhclient *       self = NM_DHCP_DHCLIENT(client);
-    NMDhcpDhclientPrivate *priv = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
-    GBytes *               client_id;
-    gs_unref_bytes GBytes *new_client_id = NULL;
+    NMDhcpDhclient           *self          = NM_DHCP_DHCLIENT(client);
+    NMDhcpDhclientPrivate    *priv          = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
+    gs_unref_bytes GBytes    *new_client_id = NULL;
+    const NMDhcpClientConfig *client_config;
 
-    client_id = nm_dhcp_client_get_client_id(client);
+    client_config = nm_dhcp_client_get_config(client);
 
     priv->conf_file = create_dhclient_config(self,
                                              AF_INET,
-                                             nm_dhcp_client_get_iface(client),
-                                             nm_dhcp_client_get_uuid(client),
-                                             client_id,
-                                             dhcp_anycast_addr,
-                                             nm_dhcp_client_get_hostname(client),
-                                             nm_dhcp_client_get_timeout(client),
-                                             nm_dhcp_client_get_use_fqdn(client),
-                                             nm_dhcp_client_get_hostname_flags(client),
-                                             nm_dhcp_client_get_mud_url(client),
-                                             nm_dhcp_client_get_reject_servers(client),
+                                             client_config->iface,
+                                             client_config->uuid,
+                                             client_config->client_id,
+                                             client_config->anycast_address,
+                                             client_config->hostname,
+                                             client_config->timeout,
+                                             client_config->use_fqdn,
+                                             client_config->hostname_flags,
+                                             client_config->mud_url,
+                                             client_config->reject_servers,
                                              &new_client_id);
     if (!priv->conf_file) {
         nm_utils_error_set_literal(error,
@@ -541,37 +543,35 @@ ip4_start(NMDhcpClient *client,
     }
 
     if (new_client_id) {
-        nm_assert(!client_id);
-        nm_dhcp_client_set_client_id(client, new_client_id);
+        nm_assert(!client_config->client_id);
+        nm_dhcp_client_set_effective_client_id(client, new_client_id);
     }
-    return dhclient_start(client, NULL, FALSE, NULL, 0, error);
+    return dhclient_start(client, NULL, FALSE, NULL, error);
 }
 
 static gboolean
-ip6_start(NMDhcpClient *            client,
-          const char *              dhcp_anycast_addr,
-          const struct in6_addr *   ll_addr,
-          NMSettingIP6ConfigPrivacy privacy,
-          guint                     needed_prefixes,
-          GError **                 error)
+ip6_start(NMDhcpClient *client, const struct in6_addr *ll_addr, GError **error)
 {
-    NMDhcpDhclient *       self = NM_DHCP_DHCLIENT(client);
-    NMDhcpDhclientPrivate *priv = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
+    NMDhcpDhclient           *self = NM_DHCP_DHCLIENT(client);
+    NMDhcpDhclientPrivate    *priv = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
+    const NMDhcpClientConfig *config;
 
-    if (nm_dhcp_client_get_iaid_explicit(client))
+    config = nm_dhcp_client_get_config(client);
+
+    if (config->v6.iaid_explicit)
         _LOGW("dhclient does not support specifying an IAID for DHCPv6, it will be ignored");
 
     priv->conf_file = create_dhclient_config(self,
                                              AF_INET6,
-                                             nm_dhcp_client_get_iface(client),
-                                             nm_dhcp_client_get_uuid(client),
+                                             config->iface,
+                                             config->uuid,
                                              NULL,
-                                             dhcp_anycast_addr,
-                                             nm_dhcp_client_get_hostname(client),
-                                             nm_dhcp_client_get_timeout(client),
+                                             config->anycast_address,
+                                             config->hostname,
+                                             config->timeout,
                                              TRUE,
-                                             nm_dhcp_client_get_hostname_flags(client),
-                                             nm_dhcp_client_get_mud_url(client),
+                                             config->hostname_flags,
+                                             config->mud_url,
                                              NULL,
                                              NULL);
     if (!priv->conf_file) {
@@ -581,18 +581,13 @@ ip6_start(NMDhcpClient *            client,
         return FALSE;
     }
 
-    return dhclient_start(client,
-                          nm_dhcp_client_get_info_only(NM_DHCP_CLIENT(self)) ? "-S" : "-N",
-                          FALSE,
-                          NULL,
-                          needed_prefixes,
-                          error);
+    return dhclient_start(client, config->v6.needed_prefixes ? "-S" : "-N", FALSE, NULL, error);
 }
 
 static void
 stop(NMDhcpClient *client, gboolean release)
 {
-    NMDhcpDhclient *       self = NM_DHCP_DHCLIENT(client);
+    NMDhcpDhclient        *self = NM_DHCP_DHCLIENT(client);
     NMDhcpDhclientPrivate *priv = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
     int                    errsv;
 
@@ -620,7 +615,7 @@ stop(NMDhcpClient *client, gboolean release)
     if (release) {
         pid_t rpid = -1;
 
-        if (dhclient_start(client, NULL, TRUE, &rpid, 0, NULL)) {
+        if (dhclient_start(client, NULL, TRUE, &rpid, NULL)) {
             /* Wait a few seconds for the release to happen */
             nm_dhcp_client_stop_pid(rpid, nm_dhcp_client_get_iface(client));
         }
@@ -630,16 +625,19 @@ stop(NMDhcpClient *client, gboolean release)
 static GBytes *
 get_duid(NMDhcpClient *client)
 {
-    NMDhcpDhclient *       self      = NM_DHCP_DHCLIENT(client);
-    NMDhcpDhclientPrivate *priv      = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
-    GBytes *               duid      = NULL;
-    gs_free char *         leasefile = NULL;
-    GError *               error     = NULL;
+    NMDhcpDhclient           *self = NM_DHCP_DHCLIENT(client);
+    NMDhcpDhclientPrivate    *priv = NM_DHCP_DHCLIENT_GET_PRIVATE(self);
+    const NMDhcpClientConfig *client_config;
+    GBytes                   *duid      = NULL;
+    gs_free char             *leasefile = NULL;
+    GError                   *error     = NULL;
+
+    client_config = nm_dhcp_client_get_config(client);
 
     /* Look in interface-specific leasefile first for backwards compat */
     leasefile = get_dhclient_leasefile(AF_INET6,
                                        nm_dhcp_client_get_iface(client),
-                                       nm_dhcp_client_get_uuid(client),
+                                       client_config->uuid,
                                        NULL);
     if (leasefile) {
         _LOGD("looking for DUID in '%s'", leasefile);
@@ -714,7 +712,7 @@ static void
 nm_dhcp_dhclient_class_init(NMDhcpDhclientClass *dhclient_class)
 {
     NMDhcpClientClass *client_class = NM_DHCP_CLIENT_CLASS(dhclient_class);
-    GObjectClass *     object_class = G_OBJECT_CLASS(dhclient_class);
+    GObjectClass      *object_class = G_OBJECT_CLASS(dhclient_class);
 
     object_class->dispose = dispose;
 
@@ -725,9 +723,10 @@ nm_dhcp_dhclient_class_init(NMDhcpDhclientClass *dhclient_class)
 }
 
 const NMDhcpClientFactory _nm_dhcp_client_factory_dhclient = {
-    .name     = "dhclient",
-    .get_type = nm_dhcp_dhclient_get_type,
-    .get_path = nm_dhcp_dhclient_get_path,
+    .name       = "dhclient",
+    .get_type_4 = nm_dhcp_dhclient_get_type,
+    .get_type_6 = nm_dhcp_dhclient_get_type,
+    .get_path   = nm_dhcp_dhclient_get_path,
 };
 
 #endif /* WITH_DHCLIENT */
