@@ -13,17 +13,18 @@
 #include <unistd.h>
 
 #include "c-list/src/c-list.h"
+#include "libnm-core-aux-intern/nm-auth-subject.h"
 
-#include "nm-setting-wireless-security.h"
 #include "nm-setting-8021x.h"
+#include "nm-setting-wireless-security.h"
+
 #include "devices/nm-device.h"
 #include "nm-active-connection.h"
+#include "nm-firewall-utils.h"
 #include "settings/nm-settings-connection.h"
-#include "nm-libnm-core-intern/nm-auth-subject.h"
 
 typedef struct {
-    CList              call_ids_lst_head;
-    NMUtilsShareRules *share_rules;
+    CList call_ids_lst_head;
 } NMActRequestPrivate;
 
 struct _NMActRequest {
@@ -71,7 +72,7 @@ nm_act_request_get_applied_connection(NMActRequest *req)
 
 struct _NMActRequestGetSecretsCallId {
     CList                       call_ids_lst;
-    NMActRequest *              self;
+    NMActRequest               *self;
     NMActRequestSecretsFunc     callback;
     gpointer                    callback_data;
     NMSettingsConnectionCallId *call_id;
@@ -90,15 +91,15 @@ _get_secrets_call_id_free(NMActRequestGetSecretsCallId *call_id)
 }
 
 static void
-get_secrets_cb(NMSettingsConnection *      connection,
+get_secrets_cb(NMSettingsConnection       *connection,
                NMSettingsConnectionCallId *call_id_s,
-               const char *                agent_username,
-               const char *                setting_name,
-               GError *                    error,
+               const char                 *agent_username,
+               const char                 *setting_name,
+               GError                     *error,
                gpointer                    user_data)
 {
     NMActRequestGetSecretsCallId *call_id = user_data;
-    NMActRequestPrivate *         priv;
+    NMActRequestPrivate          *priv;
 
     g_return_if_fail(call_id && call_id->call_id == call_id_s);
     g_return_if_fail(NM_IS_ACT_REQUEST(call_id->self));
@@ -140,19 +141,19 @@ get_secrets_cb(NMSettingsConnection *      connection,
  * Returns: a call-id.
  */
 NMActRequestGetSecretsCallId *
-nm_act_request_get_secrets(NMActRequest *               self,
+nm_act_request_get_secrets(NMActRequest                *self,
                            gboolean                     ref_self,
-                           const char *                 setting_name,
+                           const char                  *setting_name,
                            NMSecretAgentGetSecretsFlags flags,
-                           const char *const *          hints,
+                           const char *const           *hints,
                            NMActRequestSecretsFunc      callback,
                            gpointer                     callback_data)
 {
-    NMActRequestPrivate *         priv;
+    NMActRequestPrivate          *priv;
     NMActRequestGetSecretsCallId *call_id;
-    NMSettingsConnectionCallId *  call_id_s;
-    NMSettingsConnection *        settings_connection;
-    NMConnection *                applied_connection;
+    NMSettingsConnectionCallId   *call_id_s;
+    NMSettingsConnection         *settings_connection;
+    NMConnection                 *applied_connection;
 
     g_return_val_if_fail(NM_IS_ACT_REQUEST(self), NULL);
 
@@ -248,36 +249,6 @@ nm_act_request_clear_secrets(NMActRequest *self)
 
 /*****************************************************************************/
 
-NMUtilsShareRules *
-nm_act_request_get_shared(NMActRequest *req)
-{
-    g_return_val_if_fail(NM_IS_ACT_REQUEST(req), FALSE);
-
-    return NM_ACT_REQUEST_GET_PRIVATE(req)->share_rules;
-}
-
-void
-nm_act_request_set_shared(NMActRequest *req, NMUtilsShareRules *rules)
-{
-    NMActRequestPrivate *priv = NM_ACT_REQUEST_GET_PRIVATE(req);
-
-    g_return_if_fail(NM_IS_ACT_REQUEST(req));
-
-    if (priv->share_rules == rules)
-        return;
-
-    if (priv->share_rules) {
-        nm_utils_share_rules_apply(priv->share_rules, FALSE);
-        priv->share_rules = NULL;
-    }
-    if (rules) {
-        priv->share_rules = rules;
-        nm_utils_share_rules_apply(priv->share_rules, TRUE);
-    }
-}
-
-/*****************************************************************************/
-
 static void
 device_notify(GObject *object, GParamSpec *pspec, gpointer self)
 {
@@ -286,7 +257,7 @@ device_notify(GObject *object, GParamSpec *pspec, gpointer self)
 
 static void
 device_state_changed(NMActiveConnection *active,
-                     NMDevice *          device,
+                     NMDevice           *device,
                      NMDeviceState       new_state,
                      NMDeviceState       old_state)
 {
@@ -373,7 +344,7 @@ device_state_changed(NMActiveConnection *active,
 static void
 master_failed(NMActiveConnection *self)
 {
-    NMDevice *    device;
+    NMDevice     *device;
     NMDeviceState device_state;
 
     /* If the connection has an active device, fail it */
@@ -400,8 +371,8 @@ static void
 get_property(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
     NMActiveConnection *active;
-    NMDevice *          device;
-    char *              name;
+    NMDevice           *device;
+    char               *name;
 
     switch (prop_id) {
     case PROP_IP4_CONFIG:
@@ -461,14 +432,14 @@ nm_act_request_init(NMActRequest *req)
  * Returns: the new activation request on success, %NULL on error.
  */
 NMActRequest *
-nm_act_request_new(NMSettingsConnection * settings_connection,
-                   NMConnection *         applied_connection,
-                   const char *           specific_object,
-                   NMAuthSubject *        subject,
+nm_act_request_new(NMSettingsConnection  *settings_connection,
+                   NMConnection          *applied_connection,
+                   const char            *specific_object,
+                   NMAuthSubject         *subject,
                    NMActivationType       activation_type,
                    NMActivationReason     activation_reason,
                    NMActivationStateFlags initial_state_flags,
-                   NMDevice *             device)
+                   NMDevice              *device)
 {
     g_return_val_if_fail(!settings_connection || NM_IS_SETTINGS_CONNECTION(settings_connection),
                          NULL);
@@ -498,26 +469,21 @@ nm_act_request_new(NMSettingsConnection * settings_connection,
 static void
 dispose(GObject *object)
 {
-    NMActRequest *                self = NM_ACT_REQUEST(object);
-    NMActRequestPrivate *         priv = NM_ACT_REQUEST_GET_PRIVATE(self);
+    NMActRequest                 *self = NM_ACT_REQUEST(object);
+    NMActRequestPrivate          *priv = NM_ACT_REQUEST_GET_PRIVATE(self);
     NMActRequestGetSecretsCallId *call_id, *call_id_safe;
 
     /* Kill any in-progress secrets requests */
     c_list_for_each_entry_safe (call_id, call_id_safe, &priv->call_ids_lst_head, call_ids_lst)
         _do_cancel_secrets(self, call_id, TRUE);
 
-    if (priv->share_rules) {
-        nm_utils_share_rules_apply(priv->share_rules, FALSE);
-        nm_clear_pointer(&priv->share_rules, nm_utils_share_rules_free);
-    }
-
     G_OBJECT_CLASS(nm_act_request_parent_class)->dispose(object);
 }
 
 static void
 nm_act_request_class_init(NMActRequestClass *req_class)
 {
-    GObjectClass *           object_class = G_OBJECT_CLASS(req_class);
+    GObjectClass            *object_class = G_OBJECT_CLASS(req_class);
     NMActiveConnectionClass *active_class = NM_ACTIVE_CONNECTION_CLASS(req_class);
 
     /* virtual methods */
