@@ -9,7 +9,7 @@
 
 #include "nm-active-connection.h"
 #include "nm-act-request.h"
-#include "nm-libnm-core-intern/nm-auth-subject.h"
+#include "libnm-core-aux-intern/nm-auth-subject.h"
 #include "nm-core-utils.h"
 #include "nm-dbus-interface.h"
 #include "devices/nm-device.h"
@@ -22,12 +22,12 @@
 /*****************************************************************************/
 
 typedef struct {
-    char *             original_dev_path;
-    char *             original_dev_name;
+    char              *original_dev_path;
+    char              *original_dev_name;
     NMDeviceType       dev_type;
-    NMDevice *         device;
-    NMConnection *     applied_connection;
-    NMConnection *     settings_connection;
+    NMDevice          *device;
+    NMConnection      *applied_connection;
+    NMConnection      *settings_connection;
     guint64            ac_version_id;
     NMDeviceState      state;
     bool               is_software : 1;
@@ -43,14 +43,14 @@ NM_GOBJECT_PROPERTIES_DEFINE(NMCheckpoint, PROP_DEVICES, PROP_CREATED, PROP_ROLL
 struct _NMCheckpointPrivate {
     /* properties */
     GHashTable *devices;
-    GPtrArray * removed_devices;
+    GPtrArray  *removed_devices;
     gint64      created_at_ms;
     guint32     rollback_timeout_s;
     guint       timeout_id;
     /* private members */
-    NMManager *             manager;
+    NMManager              *manager;
     NMCheckpointCreateFlags flags;
-    GHashTable *            connection_uuids;
+    GHashTable             *connection_uuids;
     gulong                  dev_removed_id;
 
     NMCheckpointTimeoutCallback timeout_cb;
@@ -105,7 +105,7 @@ nm_checkpoint_log_destroy(NMCheckpoint *self)
 }
 
 void
-nm_checkpoint_set_timeout_callback(NMCheckpoint *              self,
+nm_checkpoint_set_timeout_callback(NMCheckpoint               *self,
                                    NMCheckpointTimeoutCallback callback,
                                    gpointer                    user_data)
 {
@@ -137,7 +137,7 @@ nm_checkpoint_includes_devices_of(NMCheckpoint *self, NMCheckpoint *cp_for_devic
     NMCheckpointPrivate *priv  = NM_CHECKPOINT_GET_PRIVATE(self);
     NMCheckpointPrivate *priv2 = NM_CHECKPOINT_GET_PRIVATE(cp_for_devices);
     GHashTableIter       iter;
-    NMDevice *           device;
+    NMDevice            *device;
 
     g_hash_table_iter_init(&iter, priv2->devices);
     while (g_hash_table_iter_next(&iter, (gpointer *) &device, NULL)) {
@@ -148,16 +148,16 @@ nm_checkpoint_includes_devices_of(NMCheckpoint *self, NMCheckpoint *cp_for_devic
 }
 
 static NMSettingsConnection *
-find_settings_connection(NMCheckpoint *    self,
+find_settings_connection(NMCheckpoint     *self,
                          DeviceCheckpoint *dev_checkpoint,
-                         gboolean *        need_update,
-                         gboolean *        need_activation)
+                         gboolean         *need_update,
+                         gboolean         *need_activation)
 {
-    NMCheckpointPrivate * priv = NM_CHECKPOINT_GET_PRIVATE(self);
-    NMActiveConnection *  active;
+    NMCheckpointPrivate  *priv = NM_CHECKPOINT_GET_PRIVATE(self);
+    NMActiveConnection   *active;
     NMSettingsConnection *sett_conn;
-    const char *          uuid, *ac_uuid;
-    const CList *         tmp_clist;
+    const char           *uuid, *ac_uuid;
+    const CList          *tmp_clist;
 
     *need_activation = FALSE;
     *need_update     = FALSE;
@@ -205,10 +205,10 @@ find_settings_connection(NMCheckpoint *    self,
 static gboolean
 restore_and_activate_connection(NMCheckpoint *self, DeviceCheckpoint *dev_checkpoint)
 {
-    NMCheckpointPrivate * priv = NM_CHECKPOINT_GET_PRIVATE(self);
-    NMSettingsConnection *connection;
-    gs_unref_object NMAuthSubject * subject     = NULL;
-    GError *                        local_error = NULL;
+    NMCheckpointPrivate            *priv = NM_CHECKPOINT_GET_PRIVATE(self);
+    NMSettingsConnection           *connection;
+    gs_unref_object NMAuthSubject  *subject     = NULL;
+    GError                         *local_error = NULL;
     gboolean                        need_update, need_activation;
     NMSettingsConnectionPersistMode persist_mode;
     NMSettingsConnectionIntFlags    sett_flags;
@@ -229,14 +229,16 @@ restore_and_activate_connection(NMCheckpoint *self, DeviceCheckpoint *dev_checkp
         if (need_update) {
             _LOGD("rollback: updating connection %s", nm_settings_connection_get_uuid(connection));
             persist_mode = NM_SETTINGS_CONNECTION_PERSIST_MODE_KEEP;
-            nm_settings_connection_update(connection,
-                                          dev_checkpoint->settings_connection,
-                                          persist_mode,
-                                          sett_flags,
-                                          sett_mask,
-                                          NM_SETTINGS_CONNECTION_UPDATE_REASON_NONE,
-                                          "checkpoint-rollback",
-                                          NULL);
+            nm_settings_connection_update(
+                connection,
+                dev_checkpoint->settings_connection,
+                persist_mode,
+                sett_flags,
+                sett_mask,
+                NM_SETTINGS_CONNECTION_UPDATE_REASON_RESET_SYSTEM_SECRETS
+                    | NM_SETTINGS_CONNECTION_UPDATE_REASON_UPDATE_NON_SECRET,
+                "checkpoint-rollback",
+                NULL);
         }
     } else {
         /* The connection was deleted, recreate it */
@@ -280,6 +282,11 @@ restore_and_activate_connection(NMCheckpoint *self, DeviceCheckpoint *dev_checkp
          * an internal subject. */
         if (nm_device_get_state(dev_checkpoint->device) > NM_DEVICE_STATE_DISCONNECTED
             && nm_device_get_state(dev_checkpoint->device) < NM_DEVICE_STATE_DEACTIVATING) {
+            if (!NM_FLAGS_HAS(priv->flags, NM_CHECKPOINT_CREATE_FLAG_NO_PRESERVE_EXTERNAL_PORTS)) {
+                nm_device_activation_state_set_preserve_external_ports(dev_checkpoint->device,
+                                                                       TRUE);
+            }
+
             nm_device_state_changed(dev_checkpoint->device,
                                     NM_DEVICE_STATE_DEACTIVATING,
                                     NM_DEVICE_STATE_REASON_NEW_ACTIVATION);
@@ -313,9 +320,9 @@ GVariant *
 nm_checkpoint_rollback(NMCheckpoint *self)
 {
     NMCheckpointPrivate *priv = NM_CHECKPOINT_GET_PRIVATE(self);
-    DeviceCheckpoint *   dev_checkpoint;
+    DeviceCheckpoint    *dev_checkpoint;
     GHashTableIter       iter;
-    NMDevice *           device;
+    NMDevice            *device;
     GVariantBuilder      builder;
     uint                 i;
 
@@ -422,7 +429,7 @@ next_dev:
     }
 
     if (NM_FLAGS_HAS(priv->flags, NM_CHECKPOINT_CREATE_FLAG_DELETE_NEW_CONNECTIONS)) {
-        NMSettingsConnection *con;
+        NMSettingsConnection          *con;
         gs_free NMSettingsConnection **list = NULL;
 
         g_return_val_if_fail(priv->connection_uuids, NULL);
@@ -445,7 +452,7 @@ next_dev:
     }
 
     if (NM_FLAGS_HAS(priv->flags, NM_CHECKPOINT_CREATE_FLAG_DISCONNECT_NEW_DEVICES)) {
-        const CList * tmp_lst;
+        const CList  *tmp_lst;
         NMDeviceState state;
 
         nm_manager_for_each_device (priv->manager, device, tmp_lst) {
@@ -483,7 +490,7 @@ static void
 _move_dev_to_removed_devices(NMDevice *device, NMCheckpoint *checkpoint)
 {
     NMCheckpointPrivate *priv = NM_CHECKPOINT_GET_PRIVATE(checkpoint);
-    DeviceCheckpoint *   dev_checkpoint;
+    DeviceCheckpoint    *dev_checkpoint;
 
     g_return_if_fail(device);
 
@@ -512,11 +519,11 @@ _dev_exported_changed(NMDBusObject *obj, NMCheckpoint *checkpoint)
 static DeviceCheckpoint *
 device_checkpoint_create(NMCheckpoint *checkpoint, NMDevice *device)
 {
-    DeviceCheckpoint *    dev_checkpoint;
-    NMConnection *        applied_connection;
+    DeviceCheckpoint     *dev_checkpoint;
+    NMConnection         *applied_connection;
     NMSettingsConnection *settings_connection;
-    const char *          path;
-    NMActRequest *        act_request;
+    const char           *path;
+    NMActRequest         *act_request;
 
     nm_assert(NM_IS_DEVICE(device));
     nm_assert(nm_device_is_real(device));
@@ -565,7 +572,7 @@ device_checkpoint_create(NMCheckpoint *checkpoint, NMDevice *device)
 static gboolean
 _timeout_cb(gpointer user_data)
 {
-    NMCheckpoint *       self = user_data;
+    NMCheckpoint        *self = user_data;
     NMCheckpointPrivate *priv = NM_CHECKPOINT_GET_PRIVATE(self);
 
     priv->timeout_id = 0;
@@ -617,7 +624,7 @@ nm_checkpoint_adjust_rollback_timeout(NMCheckpoint *self, guint32 add_timeout)
 static void
 get_property(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
-    NMCheckpoint *       self = NM_CHECKPOINT(object);
+    NMCheckpoint        *self = NM_CHECKPOINT(object);
     NMCheckpointPrivate *priv = NM_CHECKPOINT_GET_PRIVATE(self);
 
     switch (prop_id) {
@@ -661,13 +668,13 @@ _device_removed(NMManager *manager, NMDevice *device, gpointer user_data)
 }
 
 NMCheckpoint *
-nm_checkpoint_new(NMManager *             manager,
-                  GPtrArray *             devices,
+nm_checkpoint_new(NMManager              *manager,
+                  GPtrArray              *devices,
                   guint32                 rollback_timeout_s,
                   NMCheckpointCreateFlags flags)
 {
-    NMCheckpoint *               self;
-    NMCheckpointPrivate *        priv;
+    NMCheckpoint                *self;
+    NMCheckpointPrivate         *priv;
     NMSettingsConnection *const *con;
     gint64                       rollback_timeout_ms;
     guint                        i;
@@ -717,7 +724,7 @@ nm_checkpoint_new(NMManager *             manager,
 static void
 dispose(GObject *object)
 {
-    NMCheckpoint *       self = NM_CHECKPOINT(object);
+    NMCheckpoint        *self = NM_CHECKPOINT(object);
     NMCheckpointPrivate *priv = NM_CHECKPOINT_GET_PRIVATE(self);
 
     nm_assert(c_list_is_empty(&self->checkpoints_lst));
@@ -737,22 +744,18 @@ dispose(GObject *object)
 static const NMDBusInterfaceInfoExtended interface_info_checkpoint = {
     .parent = NM_DEFINE_GDBUS_INTERFACE_INFO_INIT(
         NM_DBUS_INTERFACE_CHECKPOINT,
-        .signals    = NM_DEFINE_GDBUS_SIGNAL_INFOS(&nm_signal_info_property_changed_legacy, ),
         .properties = NM_DEFINE_GDBUS_PROPERTY_INFOS(
-            NM_DEFINE_DBUS_PROPERTY_INFO_EXTENDED_READABLE_L("Devices",
-                                                             "ao",
-                                                             NM_CHECKPOINT_DEVICES),
-            NM_DEFINE_DBUS_PROPERTY_INFO_EXTENDED_READABLE_L("Created", "x", NM_CHECKPOINT_CREATED),
-            NM_DEFINE_DBUS_PROPERTY_INFO_EXTENDED_READABLE_L("RollbackTimeout",
-                                                             "u",
-                                                             NM_CHECKPOINT_ROLLBACK_TIMEOUT), ), ),
-    .legacy_property_changed = TRUE,
+            NM_DEFINE_DBUS_PROPERTY_INFO_EXTENDED_READABLE("Devices", "ao", NM_CHECKPOINT_DEVICES),
+            NM_DEFINE_DBUS_PROPERTY_INFO_EXTENDED_READABLE("Created", "x", NM_CHECKPOINT_CREATED),
+            NM_DEFINE_DBUS_PROPERTY_INFO_EXTENDED_READABLE("RollbackTimeout",
+                                                           "u",
+                                                           NM_CHECKPOINT_ROLLBACK_TIMEOUT), ), ),
 };
 
 static void
 nm_checkpoint_class_init(NMCheckpointClass *checkpoint_class)
 {
-    GObjectClass *     object_class      = G_OBJECT_CLASS(checkpoint_class);
+    GObjectClass      *object_class      = G_OBJECT_CLASS(checkpoint_class);
     NMDBusObjectClass *dbus_object_class = NM_DBUS_OBJECT_CLASS(checkpoint_class);
 
     g_type_class_add_private(object_class, sizeof(NMCheckpointPrivate));
