@@ -3,11 +3,17 @@
 #ifndef __NM_L3_CONFIG_DATA_H__
 #define __NM_L3_CONFIG_DATA_H__
 
-#include "nm-glib-aux/nm-dedup-multi.h"
+#include "libnm-glib-aux/nm-dedup-multi.h"
 #include "nm-setting-connection.h"
 #include "nm-setting-ip6-config.h"
-#include "platform/nm-platform.h"
-#include "platform/nmp-object.h"
+#include "libnm-platform/nm-platform.h"
+#include "libnm-platform/nmp-object.h"
+
+typedef enum {
+    NM_PROXY_CONFIG_METHOD_UNKNOWN,
+    NM_PROXY_CONFIG_METHOD_NONE,
+    NM_PROXY_CONFIG_METHOD_AUTO,
+} NMProxyConfigMethod;
 
 typedef enum {
     NM_L3_CONFIG_DAT_FLAGS_NONE = 0,
@@ -46,34 +52,28 @@ typedef enum {
 /**
  * NML3ConfigMergeFlags:
  * @NM_L3_CONFIG_MERGE_FLAGS_NONE: no flags set
- * @NM_L3_CONFIG_MERGE_FLAGS_ONLY_FOR_ACD: if this merge flag is set,
- *   the the NML3ConfigData doesn't get merged and it's information won't be
- *   synced. The only purpose is to run ACD on its IPv4 addresses, but
- *   regardless whether ACD succeeds/fails, the IP addresses won't be configured.
- *   The point is to run ACD first (without configuring it), and only
- *   commit the settings if requested. That can either happen by
- *   nm_l3cfg_add_config() the same NML3Cfg again (with a different
- *   tag), or by calling nm_l3cfg_add_config() again with this flag
- *   cleared (and the same tag).
  * @NM_L3_CONFIG_MERGE_FLAGS_NO_ROUTES: don't merge routes
  * @NM_L3_CONFIG_MERGE_FLAGS_NO_DEFAULT_ROUTES: don't merge default routes.
  *   Note that if the respective NML3ConfigData has NM_L3_CONFIG_DAT_FLAGS_IGNORE_MERGE_NO_DEFAULT_ROUTES
  *   set, this flag gets ignored during merge.
  * @NM_L3_CONFIG_MERGE_FLAGS_NO_DNS: don't merge DNS information
+ * @NM_L3_CONFIG_MERGE_FLAGS_CLONE: clone is also implemented via "merge".
+ *   In that case, it takes all settings.
  */
 typedef enum _nm_packed {
     NM_L3_CONFIG_MERGE_FLAGS_NONE              = 0,
-    NM_L3_CONFIG_MERGE_FLAGS_ONLY_FOR_ACD      = (1LL << 0),
-    NM_L3_CONFIG_MERGE_FLAGS_NO_ROUTES         = (1LL << 1),
-    NM_L3_CONFIG_MERGE_FLAGS_NO_DEFAULT_ROUTES = (1LL << 2),
-    NM_L3_CONFIG_MERGE_FLAGS_NO_DNS            = (1LL << 3),
+    NM_L3_CONFIG_MERGE_FLAGS_NO_ROUTES         = (1LL << 0),
+    NM_L3_CONFIG_MERGE_FLAGS_NO_DEFAULT_ROUTES = (1LL << 1),
+    NM_L3_CONFIG_MERGE_FLAGS_NO_DNS            = (1LL << 2),
+    NM_L3_CONFIG_MERGE_FLAGS_CLONE             = (1LL << 3),
 } NML3ConfigMergeFlags;
 
 /*****************************************************************************/
 
 static inline gboolean NM_IS_L3_CONFIG_DATA(const NML3ConfigData *self);
 
-NML3ConfigData *      nm_l3_config_data_new(NMDedupMultiIndex *multi_idx, int ifindex);
+NML3ConfigData *
+nm_l3_config_data_new(NMDedupMultiIndex *multi_idx, int ifindex, NMIPConfigSource source);
 const NML3ConfigData *nm_l3_config_data_ref(const NML3ConfigData *self);
 const NML3ConfigData *nm_l3_config_data_ref_and_seal(const NML3ConfigData *self);
 const NML3ConfigData *nm_l3_config_data_seal(const NML3ConfigData *self);
@@ -128,45 +128,51 @@ NML3ConfigData *nm_l3_config_data_new_clone(const NML3ConfigData *src, int ifind
 
 NML3ConfigData *nm_l3_config_data_new_from_connection(NMDedupMultiIndex *multi_idx,
                                                       int                ifindex,
-                                                      NMConnection *     connection,
-                                                      guint32            route_table_4,
-                                                      guint32            route_table_6,
-                                                      guint32            route_metric_4,
-                                                      guint32            route_metric_6);
+                                                      NMConnection      *connection);
 
-NML3ConfigData *nm_l3_config_data_new_from_platform(NMDedupMultiIndex *       multi_idx,
+NML3ConfigData *nm_l3_config_data_new_from_platform(NMDedupMultiIndex        *multi_idx,
                                                     int                       ifindex,
-                                                    NMPlatform *              platform,
+                                                    NMPlatform               *platform,
                                                     NMSettingIP6ConfigPrivacy ipv6_privacy_rfc4941);
 
-typedef gboolean (*NML3ConfigMergeHookAddObj)(const NML3ConfigData *l3cd,
-                                              const NMPObject *     obj,
-                                              NMTernary *           out_ip4acd_not_ready,
-                                              gpointer              user_data);
+typedef struct {
+    NMOptionBool ip4acd_not_ready;
+    NMOptionBool assume_config_once;
+    NMOptionBool force_commit;
+} NML3ConfigMergeHookResult;
 
-void nm_l3_config_data_merge(NML3ConfigData *      self,
+typedef gboolean (*NML3ConfigMergeHookAddObj)(const NML3ConfigData      *l3cd,
+                                              const NMPObject           *obj,
+                                              NML3ConfigMergeHookResult *result,
+                                              gpointer                   user_data);
+
+void nm_l3_config_data_merge(NML3ConfigData       *self,
                              const NML3ConfigData *src,
                              NML3ConfigMergeFlags  merge_flags,
                              const guint32 *default_route_table_x /* length 2, for IS_IPv4 */,
                              const guint32 *default_route_metric_x /* length 2, for IS_IPv4 */,
                              const guint32 *default_route_penalty_x /* length 2, for IS_IPv4 */,
-                             NML3ConfigMergeHookAddObj hook_add_addr,
+                             const int     *default_dns_priority_x /* length 2, for IS_IPv4 */,
+                             NML3ConfigMergeHookAddObj hook_add_obj,
                              gpointer                  hook_user_data);
 
 GPtrArray *nm_l3_config_data_get_blacklisted_ip4_routes(const NML3ConfigData *self,
                                                         gboolean              is_vrf);
 
-void nm_l3_config_data_add_dependent_routes(NML3ConfigData *self,
-                                            int             addr_family,
-                                            guint32         route_table,
-                                            guint32         route_metric,
-                                            gboolean        is_vrf);
+void nm_l3_config_data_add_dependent_onlink_routes(NML3ConfigData *self, int addr_family);
+
+void nm_l3_config_data_add_dependent_device_routes(NML3ConfigData       *self,
+                                                   int                   addr_family,
+                                                   guint32               route_table,
+                                                   guint32               route_metric,
+                                                   gboolean              force_commit,
+                                                   const NML3ConfigData *source);
 
 /*****************************************************************************/
 
 void nm_l3_config_data_log(const NML3ConfigData *self,
-                           const char *          title,
-                           const char *          prefix,
+                           const char           *title,
+                           const char           *prefix,
                            NMLogLevel            log_level,
                            NMLogDomain           log_domain);
 
@@ -183,7 +189,7 @@ NM_IS_L3_CONFIG_DATA(const NML3ConfigData *self)
      *
      * Additionally, also call nm_l3_config_data_get_ifindex(), which does more
      * checks during nm_assert(). */
-    nm_assert(nm_l3_config_data_get_ifindex(self) > 0);
+    nm_assert(!self || nm_l3_config_data_get_ifindex(self) > 0);
     return !!self;
 }
 
@@ -191,7 +197,27 @@ NMDedupMultiIndex *nm_l3_config_data_get_multi_idx(const NML3ConfigData *self);
 
 /*****************************************************************************/
 
-int nm_l3_config_data_cmp(const NML3ConfigData *a, const NML3ConfigData *b);
+typedef enum {
+    NM_L3_CONFIG_CMP_FLAGS_NONE         = 0,
+    NM_L3_CONFIG_CMP_FLAGS_IFINDEX      = (1LL << 0),
+    NM_L3_CONFIG_CMP_FLAGS_ADDRESSES_ID = (1LL << 1),
+    NM_L3_CONFIG_CMP_FLAGS_ADDRESSES    = (1LL << 2),
+    NM_L3_CONFIG_CMP_FLAGS_ROUTES_ID    = (1LL << 3),
+    NM_L3_CONFIG_CMP_FLAGS_ROUTES       = (1LL << 4),
+    NM_L3_CONFIG_CMP_FLAGS_DNS          = (1LL << 5),
+    NM_L3_CONFIG_CMP_FLAGS_OTHER        = (1LL << 6),
+    NM_L3_CONFIG_CMP_FLAGS_ALL          = (1LL << 7) - 1,
+} NML3ConfigCmpFlags;
+
+int nm_l3_config_data_cmp_full(const NML3ConfigData *a,
+                               const NML3ConfigData *b,
+                               NML3ConfigCmpFlags    flags);
+
+static inline int
+nm_l3_config_data_cmp(const NML3ConfigData *a, const NML3ConfigData *b)
+{
+    return nm_l3_config_data_cmp_full(a, b, NM_L3_CONFIG_CMP_FLAGS_ALL);
+}
 
 static inline gboolean
 nm_l3_config_data_equal(const NML3ConfigData *a, const NML3ConfigData *b)
@@ -205,15 +231,20 @@ const NMDedupMultiIdxType *nm_l3_config_data_lookup_index(const NML3ConfigData *
                                                           NMPObjectType         obj_type);
 
 const NMDedupMultiEntry *nm_l3_config_data_lookup_obj(const NML3ConfigData *self,
-                                                      const NMPObject *     obj);
+                                                      const NMPObject      *obj);
 
-const NMPlatformIP6Address *nm_l3_config_data_lookup_address_6(const NML3ConfigData * self,
+const NMPlatformIP4Address *nm_l3_config_data_lookup_address_4(const NML3ConfigData *self,
+                                                               in_addr_t             addr,
+                                                               guint8                plen,
+                                                               in_addr_t             peer_addr);
+
+const NMPlatformIP6Address *nm_l3_config_data_lookup_address_6(const NML3ConfigData  *self,
                                                                const struct in6_addr *addr);
 
 const NMDedupMultiEntry *nm_l3_config_data_lookup_route_obj(const NML3ConfigData *self,
-                                                            const NMPObject *     needle);
+                                                            const NMPObject      *needle);
 
-const NMDedupMultiEntry *nm_l3_config_data_lookup_route(const NML3ConfigData *   self,
+const NMDedupMultiEntry *nm_l3_config_data_lookup_route(const NML3ConfigData    *self,
                                                         int                      addr_family,
                                                         const NMPlatformIPRoute *needle);
 
@@ -236,6 +267,11 @@ nm_l3_config_data_lookup_routes(const NML3ConfigData *self, int addr_family)
     for (nm_dedup_multi_iter_init((iter), nm_l3_config_data_lookup_objs((self), (type))); \
          nm_platform_dedup_multi_iter_next_obj((iter), (obj), (type));)
 
+#define nm_l3_config_data_iter_ip_address_for_each(iter, self, addr_family, address)          \
+    for (nm_dedup_multi_iter_init((iter),                                                     \
+                                  nm_l3_config_data_lookup_addresses((self), (addr_family))); \
+         nm_platform_dedup_multi_iter_next_ip_address((iter), (address));)
+
 #define nm_l3_config_data_iter_ip4_address_for_each(iter, self, address)                        \
     for (nm_dedup_multi_iter_init((iter), nm_l3_config_data_lookup_addresses((self), AF_INET)); \
          nm_platform_dedup_multi_iter_next_ip4_address((iter), (address));)
@@ -244,6 +280,10 @@ nm_l3_config_data_lookup_routes(const NML3ConfigData *self, int addr_family)
     for (nm_dedup_multi_iter_init((iter), nm_l3_config_data_lookup_addresses((self), AF_INET6)); \
          nm_platform_dedup_multi_iter_next_ip6_address((iter), (address));)
 
+#define nm_l3_config_data_iter_ip_route_for_each(iter, self, addr_family, route)                   \
+    for (nm_dedup_multi_iter_init((iter), nm_l3_config_data_lookup_routes((self), (addr_family))); \
+         nm_platform_dedup_multi_iter_next_ip_route((iter), (route));)
+
 #define nm_l3_config_data_iter_ip4_route_for_each(iter, self, route)                         \
     for (nm_dedup_multi_iter_init((iter), nm_l3_config_data_lookup_routes((self), AF_INET)); \
          nm_platform_dedup_multi_iter_next_ip4_route((iter), (route));)
@@ -314,7 +354,7 @@ nmtst_l3_config_data_get_route_at_6(const NML3ConfigData *self, guint i)
 
 NML3ConfigDatFlags nm_l3_config_data_get_flags(const NML3ConfigData *self);
 
-void nm_l3_config_data_set_flags_full(NML3ConfigData *   self,
+void nm_l3_config_data_set_flags_full(NML3ConfigData    *self,
                                       NML3ConfigDatFlags flags,
                                       NML3ConfigDatFlags mask);
 
@@ -332,23 +372,21 @@ nm_l3_config_data_unset_flags(NML3ConfigData *self, NML3ConfigDatFlags flags)
 
 /*****************************************************************************/
 
-gboolean nm_l3_config_data_set_source(NML3ConfigData *self, NMIPConfigSource source);
-
 const NMPObject *nm_l3_config_data_get_first_obj(const NML3ConfigData *self,
                                                  NMPObjectType         obj_type,
                                                  gboolean (*predicate)(const NMPObject *obj));
 
-gboolean nm_l3_config_data_add_address_full(NML3ConfigData *           self,
+gboolean nm_l3_config_data_add_address_full(NML3ConfigData            *self,
                                             int                        addr_family,
-                                            const NMPObject *          obj_new,
+                                            const NMPObject           *obj_new,
                                             const NMPlatformIPAddress *pl_new,
                                             NML3ConfigAddFlags         add_flags,
-                                            const NMPObject **         out_obj_new);
+                                            const NMPObject          **out_obj_new);
 
 static inline gboolean
-nm_l3_config_data_add_address(NML3ConfigData *           self,
+nm_l3_config_data_add_address(NML3ConfigData            *self,
                               int                        addr_family,
-                              const NMPObject *          obj_new,
+                              const NMPObject           *obj_new,
                               const NMPlatformIPAddress *pl_new)
 {
     return nm_l3_config_data_add_address_full(self,
@@ -371,18 +409,18 @@ nm_l3_config_data_add_address_6(NML3ConfigData *self, const NMPlatformIP6Address
     return nm_l3_config_data_add_address(self, AF_INET6, NULL, NM_PLATFORM_IP_ADDRESS_CAST(addr));
 }
 
-gboolean nm_l3_config_data_add_route_full(NML3ConfigData *         self,
+gboolean nm_l3_config_data_add_route_full(NML3ConfigData          *self,
                                           int                      addr_family,
-                                          const NMPObject *        obj_new,
+                                          const NMPObject         *obj_new,
                                           const NMPlatformIPRoute *pl_new,
                                           NML3ConfigAddFlags       add_flags,
-                                          const NMPObject **       out_obj_new,
-                                          gboolean *               out_changed_best_default_route);
+                                          const NMPObject        **out_obj_new,
+                                          gboolean                *out_changed_best_default_route);
 
 static inline gboolean
-nm_l3_config_data_add_route(NML3ConfigData *         self,
+nm_l3_config_data_add_route(NML3ConfigData          *self,
                             int                      addr_family,
-                            const NMPObject *        obj_new,
+                            const NMPObject         *obj_new,
                             const NMPlatformIPRoute *pl_new)
 {
     return nm_l3_config_data_add_route_full(self,
@@ -409,17 +447,31 @@ nm_l3_config_data_add_route_6(NML3ConfigData *self, const NMPlatformIP6Route *rt
 const NMPObject *nm_l3_config_data_get_best_default_route(const NML3ConfigData *self,
                                                           int                   addr_family);
 
+NMSettingConnectionMdns nm_l3_config_data_get_mdns(const NML3ConfigData *self);
+
 gboolean nm_l3_config_data_set_mdns(NML3ConfigData *self, NMSettingConnectionMdns mdns);
 
+NMSettingConnectionLlmnr nm_l3_config_data_get_llmnr(const NML3ConfigData *self);
+
 gboolean nm_l3_config_data_set_llmnr(NML3ConfigData *self, NMSettingConnectionLlmnr llmnr);
 
+NMSettingConnectionDnsOverTls nm_l3_config_data_get_dns_over_tls(const NML3ConfigData *self);
+
+gboolean nm_l3_config_data_set_dns_over_tls(NML3ConfigData               *self,
+                                            NMSettingConnectionDnsOverTls dns_over_tls);
+
 NMIPRouteTableSyncMode nm_l3_config_data_get_route_table_sync(const NML3ConfigData *self,
                                                               int                   addr_family);
 
-gboolean nm_l3_config_data_set_route_table_sync(NML3ConfigData *       self,
+gboolean nm_l3_config_data_set_route_table_sync(NML3ConfigData        *self,
                                                 int                    addr_family,
                                                 NMIPRouteTableSyncMode route_table_sync);
 
+NMTernary nm_l3_config_data_get_never_default(const NML3ConfigData *self, int addr_family);
+
+gboolean
+nm_l3_config_data_set_never_default(NML3ConfigData *self, int addr_family, NMTernary never_default);
+
 NMTernary nm_l3_config_data_get_metered(const NML3ConfigData *self);
 
 gboolean nm_l3_config_data_set_metered(NML3ConfigData *self, NMTernary metered);
@@ -432,6 +484,10 @@ guint32 nm_l3_config_data_get_ip6_mtu(const NML3ConfigData *self);
 
 gboolean nm_l3_config_data_set_ip6_mtu(NML3ConfigData *self, guint32 ip6_mtu);
 
+NMUtilsIPv6IfaceId nm_l3_config_data_get_ip6_token(const NML3ConfigData *self);
+
+gboolean nm_l3_config_data_set_ip6_token(NML3ConfigData *self, NMUtilsIPv6IfaceId ipv6_token);
+
 const in_addr_t *nm_l3_config_data_get_wins(const NML3ConfigData *self, guint *out_len);
 
 gboolean nm_l3_config_data_add_wins(NML3ConfigData *self, in_addr_t wins);
@@ -439,19 +495,23 @@ gboolean nm_l3_config_data_add_wins(NML3ConfigData *self, in_addr_t wins);
 gconstpointer
 nm_l3_config_data_get_nameservers(const NML3ConfigData *self, int addr_family, guint *out_len);
 
-gboolean nm_l3_config_data_add_nameserver(NML3ConfigData *                       self,
+gboolean nm_l3_config_data_add_nameserver(NML3ConfigData                        *self,
                                           int                                    addr_family,
                                           gconstpointer /* (const NMIPAddr *) */ nameserver);
 
 gboolean nm_l3_config_data_clear_nameservers(NML3ConfigData *self, int addr_family);
 
+const in_addr_t *nm_l3_config_data_get_nis_servers(const NML3ConfigData *self, guint *out_len);
+
 gboolean nm_l3_config_data_add_nis_server(NML3ConfigData *self, in_addr_t nis_server);
 
+const char *nm_l3_config_data_get_nis_domain(const NML3ConfigData *self);
+
+gboolean nm_l3_config_data_set_nis_domain(NML3ConfigData *self, const char *nis_domain);
+
 const char *const *
 nm_l3_config_data_get_domains(const NML3ConfigData *self, int addr_family, guint *out_len);
 
-gboolean nm_l3_config_data_set_nis_domain(NML3ConfigData *self, const char *nis_domain);
-
 gboolean nm_l3_config_data_add_domain(NML3ConfigData *self, int addr_family, const char *domain);
 
 const char *const *
@@ -464,23 +524,56 @@ gboolean nm_l3_config_data_add_search(NML3ConfigData *self, int addr_family, con
 gboolean
 nm_l3_config_data_add_dns_option(NML3ConfigData *self, int addr_family, const char *dns_option);
 
+const char *const *
+nm_l3_config_data_get_dns_options(const NML3ConfigData *self, int addr_family, guint *out_len);
+
+gboolean
+nm_l3_config_data_get_dns_priority(const NML3ConfigData *self, int addr_family, int *out_prio);
+
+static inline int
+nm_l3_config_data_get_dns_priority_or_default(const NML3ConfigData *self, int addr_family)
+{
+    int v;
+
+    nm_assert_addr_family(addr_family);
+    if (!self || !nm_l3_config_data_get_dns_priority(self, addr_family, &v))
+        return 0;
+    return v;
+}
+
 gboolean
 nm_l3_config_data_set_dns_priority(NML3ConfigData *self, int addr_family, int dns_priority);
 
 NMSettingIP6ConfigPrivacy nm_l3_config_data_get_ip6_privacy(const NML3ConfigData *self);
 
-gboolean nm_l3_config_data_set_ip6_privacy(NML3ConfigData *          self,
+gboolean nm_l3_config_data_set_ip6_privacy(NML3ConfigData           *self,
                                            NMSettingIP6ConfigPrivacy ip6_privacy);
 
+NMProxyConfigMethod nm_l3_config_data_get_proxy_method(const NML3ConfigData *self);
+
+gboolean nm_l3_config_data_set_proxy_method(NML3ConfigData *self, NMProxyConfigMethod value);
+
+NMTernary nm_l3_config_data_get_proxy_browser_only(const NML3ConfigData *self);
+
+gboolean nm_l3_config_data_set_proxy_browser_only(NML3ConfigData *self, NMTernary value);
+
+const char *nm_l3_config_data_get_proxy_pac_url(const NML3ConfigData *self);
+
+gboolean nm_l3_config_data_set_proxy_pac_url(NML3ConfigData *self, const char *value);
+
+const char *nm_l3_config_data_get_proxy_pac_script(const NML3ConfigData *self);
+
+gboolean nm_l3_config_data_set_proxy_pac_script(NML3ConfigData *self, const char *value);
+
 gboolean nm_l3_config_data_get_ndisc_hop_limit(const NML3ConfigData *self, int *out_val);
 gboolean nm_l3_config_data_set_ndisc_hop_limit(NML3ConfigData *self, int val);
 
 gboolean nm_l3_config_data_get_ndisc_reachable_time_msec(const NML3ConfigData *self,
-                                                         guint32 *             out_val);
+                                                         guint32              *out_val);
 gboolean nm_l3_config_data_set_ndisc_reachable_time_msec(NML3ConfigData *self, guint32 val);
 
 gboolean nm_l3_config_data_get_ndisc_retrans_timer_msec(const NML3ConfigData *self,
-                                                        guint32 *             out_val);
+                                                        guint32              *out_val);
 gboolean nm_l3_config_data_set_ndisc_retrans_timer_msec(NML3ConfigData *self, guint32 val);
 
 struct _NMDhcpLease *nm_l3_config_data_get_dhcp_lease(const NML3ConfigData *self, int addr_family);
@@ -490,7 +583,7 @@ nm_l3_config_data_set_dhcp_lease(NML3ConfigData *self, int addr_family, struct _
 
 gboolean nm_l3_config_data_set_dhcp_lease_from_options(NML3ConfigData *self,
                                                        int             addr_family,
-                                                       GHashTable *    options_take);
+                                                       GHashTable     *options_take);
 
 static inline const NMIPAddr *
 nmtst_l3_config_data_get_best_gateway(const NML3ConfigData *self, int addr_family)
