@@ -13,12 +13,9 @@
 #include <linux/if.h>
 #include <linux/rtnetlink.h>
 
-#include "nm-utils.h"
-
-#include "nm-core-utils.h"
-#include "nm-platform/nm-platform-utils.h"
-#include "nm-platform-private.h"
-#include "nmp-object.h"
+#include "libnm-platform/nm-platform-utils.h"
+#include "libnm-platform/nm-platform-private.h"
+#include "libnm-platform/nmp-object.h"
 
 #include "nm-test-utils-core.h"
 
@@ -26,13 +23,13 @@
 
 typedef struct {
     const NMPObject *obj;
-    char *           udi;
+    char            *udi;
     struct in6_addr  ip6_lladdr;
 } NMFakePlatformLink;
 
 typedef struct {
     GHashTable *options;
-    GArray *    links;
+    GArray     *links;
 } NMFakePlatformPrivate;
 
 struct _NMFakePlatform {
@@ -63,7 +60,7 @@ G_DEFINE_TYPE(NMFakePlatform, nm_fake_platform, NM_TYPE_PLATFORM)
                                                                                               \
         if (nm_logging_enabled(__level, __domain)) {                                          \
             char              __prefix[32];                                                   \
-            const char *      __p_prefix = _NMLOG_PREFIX_NAME;                                \
+            const char       *__p_prefix = _NMLOG_PREFIX_NAME;                                \
             NMPlatform *const __self     = (self);                                            \
                                                                                               \
             if (__self && nm_platform_get_log_with_ptr(self)) {                               \
@@ -83,12 +80,12 @@ G_DEFINE_TYPE(NMFakePlatform, nm_fake_platform, NM_TYPE_PLATFORM)
 
 /*****************************************************************************/
 
-static void link_changed(NMPlatform *        platform,
+static void link_changed(NMPlatform         *platform,
                          NMFakePlatformLink *device,
                          NMPCacheOpsType     cache_op,
-                         const NMPObject *   obj_old);
+                         const NMPObject    *obj_old);
 
-static gboolean ipx_address_delete(NMPlatform *  platform,
+static gboolean ipx_address_delete(NMPlatform   *platform,
                                    int           addr_family,
                                    int           ifindex,
                                    gconstpointer addr,
@@ -98,7 +95,7 @@ static gboolean ipx_address_delete(NMPlatform *  platform,
 static gboolean
 ipx_route_delete(NMPlatform *platform, int addr_family, int ifindex, const NMPObject *obj);
 
-static gboolean ip6_address_add(NMPlatform *    platform,
+static gboolean ip6_address_add(NMPlatform     *platform,
                                 int             ifindex,
                                 struct in6_addr addr,
                                 guint8          plen,
@@ -146,7 +143,7 @@ static char *
 sysctl_get(NMPlatform *platform, const char *pathid, int dirfd, const char *path)
 {
     NMFakePlatformPrivate *priv = NM_FAKE_PLATFORM_GET_PRIVATE(platform);
-    const char *           v;
+    const char            *v;
 
     ASSERT_SYSCTL_ARGS(pathid, dirfd, path);
 
@@ -163,7 +160,7 @@ static NMFakePlatformLink *
 link_get(NMPlatform *platform, int ifindex)
 {
     NMFakePlatformPrivate *priv = NM_FAKE_PLATFORM_GET_PRIVATE(platform);
-    NMFakePlatformLink *   device;
+    NMFakePlatformLink    *device;
     int                    idx;
 
     if (ifindex <= 0)
@@ -229,11 +226,11 @@ link_add_pre(NMPlatform *platform,
              guint32     mtu)
 {
     NMFakePlatformPrivate *priv = NM_FAKE_PLATFORM_GET_PRIVATE(platform);
-    NMFakePlatformLink *   device;
+    NMFakePlatformLink    *device;
     int                    ifindex;
-    NMPObject *            o;
-    NMPlatformLink *       link;
-    gs_free char *         ip6_lladdr = NULL;
+    NMPObject             *o;
+    NMPlatformLink        *link;
+    gs_free char          *ip6_lladdr = NULL;
 
     g_assert(!name || strlen(name) < IFNAMSIZ);
 
@@ -281,27 +278,27 @@ link_add_pre(NMPlatform *platform,
 }
 
 static int
-link_add(NMPlatform *           platform,
+link_add(NMPlatform            *platform,
          NMLinkType             type,
-         const char *           name,
+         const char            *name,
          int                    parent,
-         const void *           address,
+         const void            *address,
          size_t                 address_len,
          guint32                mtu,
          gconstpointer          extra_data,
          const NMPlatformLink **out_link)
 {
-    NMFakePlatformLink * device;
-    NMFakePlatformLink * device_veth             = NULL;
+    NMFakePlatformLink             *device;
+    NMFakePlatformLink             *device_veth  = NULL;
     nm_auto_nmpobj const NMPObject *obj_old      = NULL;
     nm_auto_nmpobj const NMPObject *obj_new      = NULL;
     nm_auto_nmpobj const NMPObject *obj_old_veth = NULL;
     nm_auto_nmpobj const NMPObject *obj_new_veth = NULL;
     NMPCacheOpsType                 cache_op;
     NMPCacheOpsType                 cache_op_veth = NMP_CACHE_OPS_UNCHANGED;
-    const char *                    veth_peer     = NULL;
-    NMPObject *                     dev_obj;
-    NMPObject *                     dev_lnk = NULL;
+    const char                     *veth_peer     = NULL;
+    NMPObject                      *dev_obj;
+    NMPObject                      *dev_lnk = NULL;
 
     device = link_add_pre(platform, name, type, address, address_len, mtu);
 
@@ -392,13 +389,13 @@ static NMFakePlatformLink *
 link_add_one(NMPlatform *platform,
              const char *name,
              NMLinkType  link_type,
-             void (*prepare_fcn)(NMPlatform *        platform,
+             void (*prepare_fcn)(NMPlatform         *platform,
                                  NMFakePlatformLink *device,
                                  gconstpointer       user_data),
              gconstpointer          user_data,
              const NMPlatformLink **out_link)
 {
-    NMFakePlatformLink * device;
+    NMFakePlatformLink             *device;
     nm_auto_nmpobj const NMPObject *obj_old = NULL;
     nm_auto_nmpobj const NMPObject *obj_new = NULL;
     NMPCacheOpsType                 cache_op;
@@ -434,7 +431,7 @@ link_add_one(NMPlatform *platform,
 static gboolean
 link_delete(NMPlatform *platform, int ifindex)
 {
-    NMFakePlatformLink * device              = link_get(platform, ifindex);
+    NMFakePlatformLink             *device   = link_get(platform, ifindex);
     nm_auto_nmpobj const NMPObject *obj_old  = NULL;
     nm_auto_nmpobj const NMPObject *obj_old2 = NULL;
     NMPCacheOpsType                 cache_op;
@@ -462,10 +459,10 @@ link_delete(NMPlatform *platform, int ifindex)
 static void
 link_set_obj(NMPlatform *platform, NMFakePlatformLink *device, NMPObject *obj_tmp)
 {
-    nm_auto_nmpobj const NMPObject *obj_new = NULL;
-    nm_auto_nmpobj const NMPObject *obj_old = NULL;
-    nm_auto_nmpobj NMPObject *obj_tmp_tmp   = NULL;
-    NMPCacheOpsType           cache_op;
+    nm_auto_nmpobj const NMPObject *obj_new     = NULL;
+    nm_auto_nmpobj const NMPObject *obj_old     = NULL;
+    nm_auto_nmpobj NMPObject       *obj_tmp_tmp = NULL;
+    NMPCacheOpsType                 cache_op;
 
     g_assert(device);
     g_assert(NMP_OBJECT_GET_TYPE(device->obj) == NMP_OBJECT_TYPE_LINK);
@@ -506,11 +503,26 @@ link_set_flags(NMPlatform *platform, NMFakePlatformLink *device, guint n_ifi_fla
     link_set_obj(platform, device, obj_tmp);
 }
 
+static int
+link_change_flags(NMPlatform *platform, int ifindex, unsigned flags_mask, unsigned flags_set)
+{
+    NMFakePlatformLink *device = link_get(platform, ifindex);
+
+    if (!device)
+        return -ENOENT;
+
+    link_set_flags(platform,
+                   device,
+                   NM_FLAGS_ASSIGN_MASK(device->obj->link.n_ifi_flags, flags_mask, flags_set));
+
+    return 0;
+}
+
 static void
-link_changed(NMPlatform *        platform,
+link_changed(NMPlatform         *platform,
              NMFakePlatformLink *device,
              NMPCacheOpsType     cache_op,
-             const NMPObject *   obj_old)
+             const NMPObject    *obj_old)
 {
     g_assert(device->obj);
 
@@ -543,72 +555,13 @@ link_changed(NMPlatform *        platform,
     }
 }
 
-static gboolean
-link_set_up(NMPlatform *platform, int ifindex, gboolean *out_no_firmware)
-{
-    NMFakePlatformLink *device = link_get(platform, ifindex);
-
-    if (out_no_firmware)
-        *out_no_firmware = FALSE;
-
-    if (!device) {
-        _LOGE("failure changing link: netlink error (No such device)");
-        return FALSE;
-    }
-
-    link_set_flags(platform, device, NM_FLAGS_ASSIGN(device->obj->link.n_ifi_flags, IFF_UP, TRUE));
-    return TRUE;
-}
-
-static gboolean
-link_set_down(NMPlatform *platform, int ifindex)
-{
-    NMFakePlatformLink *device = link_get(platform, ifindex);
-
-    if (!device) {
-        _LOGE("failure changing link: netlink error (No such device)");
-        return FALSE;
-    }
-
-    link_set_flags(platform, device, NM_FLAGS_UNSET(device->obj->link.n_ifi_flags, IFF_UP));
-    return TRUE;
-}
-
-static gboolean
-link_set_arp(NMPlatform *platform, int ifindex)
-{
-    NMFakePlatformLink *device = link_get(platform, ifindex);
-
-    if (!device) {
-        _LOGE("failure changing link: netlink error (No such device)");
-        return FALSE;
-    }
-
-    link_set_flags(platform, device, NM_FLAGS_UNSET(device->obj->link.n_ifi_flags, IFF_NOARP));
-    return TRUE;
-}
-
-static gboolean
-link_set_noarp(NMPlatform *platform, int ifindex)
-{
-    NMFakePlatformLink *device = link_get(platform, ifindex);
-
-    if (!device) {
-        _LOGE("failure changing link: netlink error (No such device)");
-        return FALSE;
-    }
-
-    link_set_flags(platform, device, NM_FLAGS_SET(device->obj->link.n_ifi_flags, IFF_NOARP));
-    return TRUE;
-}
-
 static int
 link_set_address(NMPlatform *platform, int ifindex, gconstpointer addr, size_t len)
 {
-    NMFakePlatformLink *device        = link_get(platform, ifindex);
+    NMFakePlatformLink       *device  = link_get(platform, ifindex);
     nm_auto_nmpobj NMPObject *obj_tmp = NULL;
 
-    if (len == 0 || len > NM_UTILS_HWADDR_LEN_MAX || !addr)
+    if (len == 0 || len > _NM_UTILS_HWADDR_LEN_MAX || !addr)
         g_return_val_if_reached(-NME_BUG);
 
     if (!device)
@@ -626,7 +579,7 @@ link_set_address(NMPlatform *platform, int ifindex, gconstpointer addr, size_t l
 static int
 link_set_mtu(NMPlatform *platform, int ifindex, guint32 mtu)
 {
-    NMFakePlatformLink *device        = link_get(platform, ifindex);
+    NMFakePlatformLink       *device  = link_get(platform, ifindex);
     nm_auto_nmpobj NMPObject *obj_tmp = NULL;
 
     if (!device) {
@@ -643,9 +596,9 @@ link_set_mtu(NMPlatform *platform, int ifindex, guint32 mtu)
 static gboolean
 link_get_driver_info(NMPlatform *platform,
                      int         ifindex,
-                     char **     out_driver_name,
-                     char **     out_driver_version,
-                     char **     out_fw_version)
+                     char      **out_driver_name,
+                     char      **out_driver_version,
+                     char      **out_fw_version)
 {
     if (out_driver_name)
         *out_driver_name = NULL;
@@ -730,8 +683,8 @@ link_enslave(NMPlatform *platform, int master, int slave)
 static gboolean
 link_release(NMPlatform *platform, int master_idx, int slave_idx)
 {
-    NMFakePlatformLink *master        = link_get(platform, master_idx);
-    NMFakePlatformLink *slave         = link_get(platform, slave_idx);
+    NMFakePlatformLink       *master  = link_get(platform, master_idx);
+    NMFakePlatformLink       *slave   = link_get(platform, slave_idx);
     nm_auto_nmpobj NMPObject *obj_tmp = NULL;
 
     g_return_val_if_fail(master, FALSE);
@@ -747,10 +700,10 @@ link_release(NMPlatform *platform, int master_idx, int slave_idx)
 }
 
 static gboolean
-link_vlan_change(NMPlatform *            platform,
+link_vlan_change(NMPlatform             *platform,
                  int                     ifindex,
-                 NMVlanFlags             flags_mask,
-                 NMVlanFlags             flags_set,
+                 _NMVlanFlags            flags_mask,
+                 _NMVlanFlags            flags_set,
                  gboolean                ingress_reset_all,
                  const NMVlanQosMapping *ingress_map,
                  gsize                   n_ingress_map,
@@ -770,8 +723,8 @@ static void
 _infiniband_add_prepare(NMPlatform *platform, NMFakePlatformLink *device, gconstpointer user_data)
 {
     const struct infiniband_add_data *d = user_data;
-    NMPObject *                       obj_tmp;
-    NMPObject *                       lnk;
+    NMPObject                        *obj_tmp;
+    NMPObject                        *lnk;
 
     obj_tmp = (NMPObject *) device->obj;
 
@@ -784,12 +737,12 @@ _infiniband_add_prepare(NMPlatform *platform, NMFakePlatformLink *device, gconst
 }
 
 static gboolean
-infiniband_partition_add(NMPlatform *           platform,
+infiniband_partition_add(NMPlatform            *platform,
                          int                    parent,
                          int                    p_key,
                          const NMPlatformLink **out_link)
 {
-    NMFakePlatformLink *             parent_device;
+    NMFakePlatformLink              *parent_device;
     char                             name[IFNAMSIZ];
     const struct infiniband_add_data d = {
         .parent = parent,
@@ -799,7 +752,7 @@ infiniband_partition_add(NMPlatform *           platform,
     parent_device = link_get(platform, parent);
     g_return_val_if_fail(parent_device != NULL, FALSE);
 
-    nm_utils_new_infiniband_name(name, parent_device->obj->link.name, p_key);
+    nmp_utils_new_infiniband_name(name, parent_device->obj->link.name, p_key);
 
     link_add_one(platform, name, NM_LINK_TYPE_INFINIBAND, _infiniband_add_prepare, &d, out_link);
     return TRUE;
@@ -809,17 +762,17 @@ static gboolean
 infiniband_partition_delete(NMPlatform *platform, int parent, int p_key)
 {
     NMFakePlatformLink *parent_device;
-    gs_free char *      name = NULL;
+    gs_free char       *name = NULL;
 
     parent_device = link_get(platform, parent);
     g_return_val_if_fail(parent_device != NULL, FALSE);
 
-    nm_utils_new_infiniband_name(name, parent_device->obj->link.name, p_key);
+    nmp_utils_new_infiniband_name(name, parent_device->obj->link.name, p_key);
     return link_delete(platform, nm_platform_link_get_ifindex(platform, name));
 }
 
 static gboolean
-wifi_get_capabilities(NMPlatform *platform, int ifindex, NMDeviceWifiCapabilities *caps)
+wifi_get_capabilities(NMPlatform *platform, int ifindex, _NMDeviceWifiCapabilities *caps)
 {
     NMFakePlatformLink *device = link_get(platform, ifindex);
 
@@ -829,10 +782,10 @@ wifi_get_capabilities(NMPlatform *platform, int ifindex, NMDeviceWifiCapabilitie
         return FALSE;
 
     if (caps) {
-        *caps = (NM_WIFI_DEVICE_CAP_CIPHER_WEP40 | NM_WIFI_DEVICE_CAP_CIPHER_WEP104
-                 | NM_WIFI_DEVICE_CAP_CIPHER_TKIP | NM_WIFI_DEVICE_CAP_CIPHER_CCMP
-                 | NM_WIFI_DEVICE_CAP_WPA | NM_WIFI_DEVICE_CAP_RSN | NM_WIFI_DEVICE_CAP_AP
-                 | NM_WIFI_DEVICE_CAP_ADHOC);
+        *caps = (_NM_WIFI_DEVICE_CAP_CIPHER_WEP40 | _NM_WIFI_DEVICE_CAP_CIPHER_WEP104
+                 | _NM_WIFI_DEVICE_CAP_CIPHER_TKIP | _NM_WIFI_DEVICE_CAP_CIPHER_CCMP
+                 | _NM_WIFI_DEVICE_CAP_WPA | _NM_WIFI_DEVICE_CAP_RSN | _NM_WIFI_DEVICE_CAP_AP
+                 | _NM_WIFI_DEVICE_CAP_ADHOC);
     }
     return TRUE;
 }
@@ -861,14 +814,14 @@ wifi_get_rate(NMPlatform *platform, int ifindex)
     return 0;
 }
 
-static NM80211Mode
+static _NM80211Mode
 wifi_get_mode(NMPlatform *platform, int ifindex)
 {
-    return NM_802_11_MODE_UNKNOWN;
+    return _NM_802_11_MODE_UNKNOWN;
 }
 
 static void
-wifi_set_mode(NMPlatform *platform, int ifindex, NM80211Mode mode)
+wifi_set_mode(NMPlatform *platform, int ifindex, _NM80211Mode mode)
 {
     ;
 }
@@ -906,11 +859,11 @@ mesh_set_ssid(NMPlatform *platform, int ifindex, const guint8 *ssid, gsize len)
 static gboolean
 ipx_address_add(NMPlatform *platform, int addr_family, const NMPlatformObject *address)
 {
-    nm_auto_nmpobj NMPObject *obj = NULL;
-    NMPCacheOpsType           cache_op;
+    nm_auto_nmpobj NMPObject       *obj = NULL;
+    NMPCacheOpsType                 cache_op;
     nm_auto_nmpobj const NMPObject *obj_old = NULL;
     nm_auto_nmpobj const NMPObject *obj_new = NULL;
-    NMPCache *                      cache   = nm_platform_get_cache(platform);
+    NMPCache                       *cache   = nm_platform_get_cache(platform);
 
     g_assert(NM_IN_SET(addr_family, AF_INET, AF_INET6));
 
@@ -957,7 +910,7 @@ ip4_address_add(NMPlatform *platform,
 }
 
 static gboolean
-ip6_address_add(NMPlatform *    platform,
+ip6_address_add(NMPlatform     *platform,
                 int             ifindex,
                 struct in6_addr addr,
                 guint8          plen,
@@ -985,7 +938,7 @@ ip6_address_add(NMPlatform *    platform,
 }
 
 static gboolean
-ipx_address_delete(NMPlatform *  platform,
+ipx_address_delete(NMPlatform   *platform,
                    int           addr_family,
                    int           ifindex,
                    gconstpointer addr,
@@ -1004,11 +957,10 @@ ipx_address_delete(NMPlatform *  platform,
     peer_addr_i = peer_addr ? *((guint32 *) peer_addr) : 0;
 
     nmp_cache_iter_for_each (&iter,
-                             nm_platform_lookup_object(platform,
-                                                       addr_family == AF_INET
-                                                           ? NMP_OBJECT_TYPE_IP4_ADDRESS
-                                                           : NMP_OBJECT_TYPE_IP6_ADDRESS,
-                                                       0),
+                             nm_platform_lookup_obj_type(platform,
+                                                         addr_family == AF_INET
+                                                             ? NMP_OBJECT_TYPE_IP4_ADDRESS
+                                                             : NMP_OBJECT_TYPE_IP6_ADDRESS),
                              &o) {
         const NMPObject *obj_old = NULL;
 
@@ -1132,23 +1084,23 @@ object_delete(NMPlatform *platform, const NMPObject *obj)
 }
 
 static int
-ip_route_add(NMPlatform *             platform,
+ip_route_add(NMPlatform              *platform,
              NMPNlmFlags              flags,
              int                      addr_family,
              const NMPlatformIPRoute *route)
 {
-    NMDedupMultiIter iter;
-    nm_auto_nmpobj NMPObject *obj = NULL;
-    NMPCacheOpsType           cache_op;
-    const NMPObject *         o                 = NULL;
+    NMDedupMultiIter                iter;
+    nm_auto_nmpobj NMPObject       *obj = NULL;
+    NMPCacheOpsType                 cache_op;
+    const NMPObject                *o           = NULL;
     nm_auto_nmpobj const NMPObject *obj_old     = NULL;
     nm_auto_nmpobj const NMPObject *obj_new     = NULL;
     nm_auto_nmpobj const NMPObject *obj_replace = NULL;
-    NMPCache *                      cache       = nm_platform_get_cache(platform);
+    NMPCache                       *cache       = nm_platform_get_cache(platform);
     gboolean                        has_gateway = FALSE;
-    NMPlatformIPRoute *             r           = NULL;
-    NMPlatformIP4Route *            r4          = NULL;
-    NMPlatformIP6Route *            r6          = NULL;
+    NMPlatformIPRoute              *r           = NULL;
+    NMPlatformIP4Route             *r4          = NULL;
+    NMPlatformIP6Route             *r6          = NULL;
     gboolean                        has_same_weak_id;
     gboolean                        only_dirty;
     guint16                         nlmsgflags;
@@ -1164,6 +1116,7 @@ ip_route_add(NMPlatform *             platform,
                                                 : NMP_OBJECT_TYPE_IP6_ROUTE,
                          (const NMPlatformObject *) route);
     r = NMP_OBJECT_CAST_IP_ROUTE(obj);
+
     nm_platform_ip_route_normalize(addr_family, r);
 
     switch (addr_family) {
@@ -1185,7 +1138,7 @@ ip_route_add(NMPlatform *             platform,
         gboolean has_route_to_gw = FALSE;
 
         nmp_cache_iter_for_each (&iter,
-                                 nm_platform_lookup_object(platform, NMP_OBJECT_GET_TYPE(obj), 0),
+                                 nm_platform_lookup_obj_type(platform, NMP_OBJECT_GET_TYPE(obj)),
                                  &o) {
             if (addr_family == AF_INET) {
                 const NMPlatformIP4Route *item = NMP_OBJECT_CAST_IP4_ROUTE(o);
@@ -1342,8 +1295,8 @@ finalize(GObject *object)
 static void
 nm_fake_platform_class_init(NMFakePlatformClass *klass)
 {
-    GObjectClass *              object_class   = G_OBJECT_CLASS(klass);
-    NMPlatformClass *           platform_class = NM_PLATFORM_CLASS(klass);
+    GObjectClass               *object_class   = G_OBJECT_CLASS(klass);
+    NMPlatformClass            *platform_class = NM_PLATFORM_CLASS(klass);
     NMPlatformKernelSupportType kernel_support;
 
     for (kernel_support = 0; kernel_support < _NM_PLATFORM_KERNEL_SUPPORT_NUM; kernel_support++)
@@ -1357,14 +1310,11 @@ nm_fake_platform_class_init(NMFakePlatformClass *klass)
     platform_class->link_add    = link_add;
     platform_class->link_delete = link_delete;
 
-    platform_class->link_set_up    = link_set_up;
-    platform_class->link_set_down  = link_set_down;
-    platform_class->link_set_arp   = link_set_arp;
-    platform_class->link_set_noarp = link_set_noarp;
-
     platform_class->link_set_address = link_set_address;
     platform_class->link_set_mtu     = link_set_mtu;
 
+    platform_class->link_change_flags = link_change_flags;
+
     platform_class->link_get_driver_info = link_get_driver_info;
 
     platform_class->link_supports_carrier_detect = link_supports_carrier_detect;
