@@ -12,7 +12,8 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-#include "nm-core-internal.h"
+#include "libnm-glib-aux/nm-uuid.h"
+#include "libnm-core-intern/nm-core-internal.h"
 #include "settings/nm-settings-plugin.h"
 
 #include "nms-ifupdown-plugin.h"
@@ -32,14 +33,6 @@
 
 /*****************************************************************************/
 
-#define _str_has_prefix(val, prefix, require_suffix)                   \
-    ({                                                                 \
-        const char *_val = (val);                                      \
-                                                                       \
-        (strncmp(_val, "" prefix "", NM_STRLEN(prefix)) == 0)          \
-            && (!(require_suffix) || _val[NM_STRLEN(prefix)] != '\0'); \
-    })
-
 static const char *
 _ifupdownplugin_guess_connection_type(if_block *block)
 {
@@ -51,8 +44,7 @@ _ifupdownplugin_guess_connection_type(if_block *block)
         if_data *ifb;
 
         c_list_for_each_entry (ifb, &block->data_lst_head, data_lst) {
-            if (_str_has_prefix(ifb->key, "wireless-", FALSE)
-                || _str_has_prefix(ifb->key, "wpa-", FALSE)) {
+            if (NM_STR_HAS_PREFIX(ifb->key, "wireless-") || NM_STR_HAS_PREFIX(ifb->key, "wpa-")) {
                 ret_type = NM_SETTING_WIRELESS_SETTING_NAME;
                 break;
             }
@@ -66,7 +58,7 @@ _ifupdownplugin_guess_connection_type(if_block *block)
 }
 
 struct _Mapping {
-    const char *   domain;
+    const char    *domain;
     const gpointer target;
 };
 
@@ -86,8 +78,8 @@ map_by_mapping(struct _Mapping *mapping, const char *key)
 static void
 update_wireless_setting_from_if_block(NMConnection *connection, if_block *block)
 {
-    if_data *       curr;
-    const char *    value     = ifparser_getkey(block, "inet");
+    if_data        *curr;
+    const char     *value     = ifparser_getkey(block, "inet");
     struct _Mapping mapping[] = {{"ssid", "ssid"},
                                  {"essid", "ssid"},
                                  {"mode", "mode"},
@@ -102,7 +94,7 @@ update_wireless_setting_from_if_block(NMConnection *connection, if_block *block)
     wireless_setting = NM_SETTING_WIRELESS(nm_setting_wireless_new());
 
     c_list_for_each_entry (curr, &block->data_lst_head, data_lst) {
-        if (_str_has_prefix(curr->key, "wireless-", TRUE)) {
+        if (NM_STR_HAS_PREFIX_WITH_MORE(curr->key, "wireless-")) {
             const char *newkey = map_by_mapping(mapping, curr->key + NM_STRLEN("wireless-"));
 
             _LOGI("wireless setting key: %s='%s'", newkey, curr->data);
@@ -137,7 +129,7 @@ update_wireless_setting_from_if_block(NMConnection *connection, if_block *block)
             } else {
                 g_object_set(wireless_setting, newkey, curr->data, NULL);
             }
-        } else if (_str_has_prefix(curr->key, "wpa-", TRUE)) {
+        } else if (NM_STR_HAS_PREFIX_WITH_MORE(curr->key, "wpa-")) {
             const char *newkey = map_by_mapping(mapping, curr->key + NM_STRLEN("wpa-"));
 
             if (nm_streq0(newkey, "ssid")) {
@@ -218,7 +210,7 @@ static gpointer
 string_to_glist_of_strings(const char *data)
 {
     GSList *ret    = NULL;
-    char *  string = (char *) data;
+    char   *string = (char *) data;
     while (string) {
         char *next = NULL;
         if ((next = strchr(string, ' ')) || (next = strchr(string, '\t'))
@@ -245,8 +237,8 @@ slist_free_all(gpointer slist)
 static void
 update_wireless_security_setting_from_if_block(NMConnection *connection, if_block *block)
 {
-    if_data *       curr;
-    const char *    value     = ifparser_getkey(block, "inet");
+    if_data        *curr;
+    const char     *value     = ifparser_getkey(block, "inet");
     struct _Mapping mapping[] = {{"psk", "psk"},
                                  {"identity", "leap-username"},
                                  {"password", "leap-password"},
@@ -291,7 +283,7 @@ update_wireless_security_setting_from_if_block(NMConnection *connection, if_bloc
                                            {NULL, NULL}};
 
     NMSettingWirelessSecurity *wireless_security_setting;
-    NMSettingWireless *        s_wireless;
+    NMSettingWireless         *s_wireless;
     gboolean                   security = FALSE;
 
     if (nm_streq0(value, "ppp"))
@@ -304,11 +296,11 @@ update_wireless_security_setting_from_if_block(NMConnection *connection, if_bloc
     wireless_security_setting = NM_SETTING_WIRELESS_SECURITY(nm_setting_wireless_security_new());
 
     c_list_for_each_entry (curr, &block->data_lst_head, data_lst) {
-        if (_str_has_prefix(curr->key, "wireless-", TRUE)) {
-            const char *          key                  = curr->key + NM_STRLEN("wireless-");
-            char *                property_value       = NULL;
+        if (NM_STR_HAS_PREFIX_WITH_MORE(curr->key, "wireless-")) {
+            const char           *key                  = curr->key + NM_STRLEN("wireless-");
+            char                 *property_value       = NULL;
             gpointer              typed_property_value = NULL;
-            const char *          newkey               = map_by_mapping(mapping, key);
+            const char           *newkey               = map_by_mapping(mapping, key);
             IfupdownStrDupeFunc   dupe_func            = map_by_mapping(dupe_mapping, key);
             IfupdownStrToTypeFunc type_map_func        = map_by_mapping(type_mapping, key);
             GFreeFunc             free_func            = map_by_mapping(free_type_mapping, key);
@@ -336,11 +328,11 @@ wireless_next:
             if (typed_property_value && free_func)
                 (*free_func)(typed_property_value);
 
-        } else if (_str_has_prefix(curr->key, "wpa-", TRUE)) {
-            const char *          key                  = curr->key + NM_STRLEN("wpa-");
-            char *                property_value       = NULL;
+        } else if (NM_STR_HAS_PREFIX_WITH_MORE(curr->key, "wpa-")) {
+            const char           *key                  = curr->key + NM_STRLEN("wpa-");
+            char                 *property_value       = NULL;
             gpointer              typed_property_value = NULL;
-            const char *          newkey               = map_by_mapping(mapping, key);
+            const char           *newkey               = map_by_mapping(mapping, key);
             IfupdownStrDupeFunc   dupe_func            = map_by_mapping(dupe_mapping, key);
             IfupdownStrToTypeFunc type_map_func        = map_by_mapping(type_mapping, key);
             GFreeFunc             free_func            = map_by_mapping(free_type_mapping, key);
@@ -399,13 +391,13 @@ static void
 ifupdown_ip4_add_dns(NMSettingIPConfig *s_ip4, const char *dns)
 {
     gs_free const char **list = NULL;
-    const char **        iter;
+    const char         **iter;
     guint32              addr;
 
     if (dns == NULL)
         return;
 
-    list = nm_utils_strsplit_set(dns, " \t");
+    list = nm_strsplit_set(dns, " \t");
     for (iter = list; iter && *iter; iter++) {
         if (!inet_pton(AF_INET, *iter, &addr)) {
             _LOGW("    ignoring invalid nameserver '%s'", *iter);
@@ -421,19 +413,19 @@ static gboolean
 update_ip4_setting_from_if_block(NMConnection *connection, if_block *block, GError **error)
 {
     gs_unref_object NMSettingIPConfig *s_ip4 = NM_SETTING_IP_CONFIG(nm_setting_ip4_config_new());
-    const char *                       type  = ifparser_getkey(block, "inet");
+    const char                        *type  = ifparser_getkey(block, "inet");
 
     if (!nm_streq0(type, "static")) {
         g_object_set(s_ip4, NM_SETTING_IP_CONFIG_METHOD, NM_SETTING_IP4_CONFIG_METHOD_AUTO, NULL);
     } else {
         guint32      tmp_mask;
         NMIPAddress *addr;
-        const char * address_v;
-        const char * netmask_v;
-        const char * gateway_v;
-        const char * nameserver_v;
-        const char * nameservers_v;
-        const char * search_v;
+        const char  *address_v;
+        const char  *netmask_v;
+        const char  *gateway_v;
+        const char  *nameserver_v;
+        const char  *nameservers_v;
+        const char  *search_v;
         guint32      netmask_int = 32;
 
         /* Address */
@@ -503,9 +495,9 @@ update_ip4_setting_from_if_block(NMConnection *connection, if_block *block, GErr
         search_v = ifparser_getkey(block, "dns-search");
         if (search_v) {
             gs_free const char **list = NULL;
-            const char **        iter;
+            const char         **iter;
 
-            list = nm_utils_strsplit_set(search_v, " \t");
+            list = nm_strsplit_set(search_v, " \t");
             for (iter = list; iter && *iter; iter++) {
                 if (!nm_setting_ip_config_add_dns_search(s_ip4, *iter))
                     _LOGW("    duplicate DNS domain '%s'", *iter);
@@ -523,13 +515,13 @@ static void
 ifupdown_ip6_add_dns(NMSettingIPConfig *s_ip6, const char *dns)
 {
     gs_free const char **list = NULL;
-    const char **        iter;
+    const char         **iter;
     struct in6_addr      addr;
 
     if (dns == NULL)
         return;
 
-    list = nm_utils_strsplit_set(dns, " \t");
+    list = nm_strsplit_set(dns, " \t");
     for (iter = list; iter && *iter; iter++) {
         if (!inet_pton(AF_INET6, *iter, &addr)) {
             _LOGW("    ignoring invalid nameserver '%s'", *iter);
@@ -545,18 +537,18 @@ static gboolean
 update_ip6_setting_from_if_block(NMConnection *connection, if_block *block, GError **error)
 {
     gs_unref_object NMSettingIPConfig *s_ip6 = NM_SETTING_IP_CONFIG(nm_setting_ip6_config_new());
-    const char *                       type  = ifparser_getkey(block, "inet6");
+    const char                        *type  = ifparser_getkey(block, "inet6");
 
     if (!NM_IN_STRSET(type, "static", "v4tunnel")) {
         g_object_set(s_ip6, NM_SETTING_IP_CONFIG_METHOD, NM_SETTING_IP6_CONFIG_METHOD_AUTO, NULL);
     } else {
         NMIPAddress *addr;
-        const char * address_v;
-        const char * prefix_v;
-        const char * gateway_v;
-        const char * nameserver_v;
-        const char * nameservers_v;
-        const char * search_v;
+        const char  *address_v;
+        const char  *prefix_v;
+        const char  *gateway_v;
+        const char  *nameserver_v;
+        const char  *nameservers_v;
+        const char  *search_v;
         guint        prefix_int;
 
         address_v = ifparser_getkey(block, "address");
@@ -611,9 +603,9 @@ update_ip6_setting_from_if_block(NMConnection *connection, if_block *block, GErr
         search_v = ifparser_getkey(block, "dns-search");
         if (search_v) {
             gs_free const char **list = NULL;
-            const char **        iter;
+            const char         **iter;
 
-            list = nm_utils_strsplit_set(search_v, " \t");
+            list = nm_strsplit_set(search_v, " \t");
             for (iter = list; iter && *iter; iter++) {
                 if (!nm_setting_ip_config_add_dns_search(s_ip6, *iter))
                     _LOGW("    duplicate DNS domain '%s'", *iter);
@@ -631,10 +623,10 @@ NMConnection *
 ifupdown_new_connection_from_if_block(if_block *block, gboolean autoconnect, GError **error)
 {
     gs_unref_object NMConnection *connection = NULL;
-    const char *                  type;
-    gs_free char *                idstr = NULL;
-    gs_free char *                uuid  = NULL;
-    NMSettingConnection *         s_con;
+    const char                   *type;
+    gs_free char                 *idstr = NULL;
+    gs_free char                 *uuid  = NULL;
+    NMSettingConnection          *s_con;
 
     connection = nm_simple_connection_new();
 
@@ -644,7 +636,7 @@ ifupdown_new_connection_from_if_block(if_block *block, gboolean autoconnect, GEr
     type  = _ifupdownplugin_guess_connection_type(block);
     idstr = g_strconcat("Ifupdown (", block->name, ")", NULL);
 
-    uuid = nm_utils_uuid_generate_from_string(idstr, -1, NM_UTILS_UUID_TYPE_LEGACY, NULL);
+    uuid = nm_uuid_generate_from_string_str(idstr, -1, NM_UUID_TYPE_LEGACY, NULL);
     g_object_set(s_con,
                  NM_SETTING_CONNECTION_TYPE,
                  type,
@@ -655,7 +647,7 @@ ifupdown_new_connection_from_if_block(if_block *block, gboolean autoconnect, GEr
                  NM_SETTING_CONNECTION_UUID,
                  uuid,
                  NM_SETTING_CONNECTION_AUTOCONNECT,
-                 (gboolean)(!!autoconnect),
+                 (gboolean) (!!autoconnect),
                  NULL);
 
     _LOGD("update_connection_setting_from_if_block: name:%s, type:%s, id:%s, uuid: %s",
