@@ -8,27 +8,27 @@
 
 #include "nm-supplicant-manager.h"
 
-#include "nm-core-internal.h"
+#include "libnm-core-intern/nm-core-internal.h"
 #include "nm-dbus-manager.h"
-#include "nm-glib-aux/nm-dbus-aux.h"
-#include "nm-glib-aux/nm-ref-string.h"
+#include "libnm-glib-aux/nm-dbus-aux.h"
+#include "libnm-glib-aux/nm-ref-string.h"
 #include "nm-supplicant-interface.h"
 #include "nm-supplicant-types.h"
-#include "platform/nm-platform.h"
+#include "libnm-platform/nm-platform.h"
 
 /*****************************************************************************/
 
 #define CREATE_IFACE_TRY_COUNT_MAX 7u
 
 struct _NMSupplMgrCreateIfaceHandle {
-    NMSupplicantManager *                self;
+    NMSupplicantManager                 *self;
     CList                                create_iface_lst;
-    GCancellable *                       cancellable;
+    GCancellable                        *cancellable;
     NMSupplicantManagerCreateInterfaceCb callback;
     gpointer                             callback_user_data;
-    NMShutdownWaitObjHandle *            shutdown_handle;
-    NMRefString *                        name_owner;
-    GError *                             fail_on_idle_error;
+    NMShutdownWaitObjHandle             *shutdown_handle;
+    NMRefString                         *name_owner;
+    GError                              *fail_on_idle_error;
     NMSupplicantDriver                   driver;
     int                                  ifindex;
     guint                                fail_on_idle_id;
@@ -98,19 +98,19 @@ NM_CACHED_QUARK_FCN("nm-supplicant-error-quark", nm_supplicant_error_quark);
 /*****************************************************************************/
 
 static void     _create_iface_proceed_all(NMSupplicantManager *self, GError *error);
-static void     _supp_iface_add(NMSupplicantManager *  self,
-                                NMRefString *          iface_path,
+static void     _supp_iface_add(NMSupplicantManager   *self,
+                                NMRefString           *iface_path,
                                 NMSupplicantInterface *supp_iface);
-static void     _supp_iface_remove_one(NMSupplicantManager *  self,
+static void     _supp_iface_remove_one(NMSupplicantManager   *self,
                                        NMSupplicantInterface *supp_iface,
                                        gboolean               force_remove_from_supplicant,
-                                       const char *           reason);
-static void     _create_iface_dbus_call_get_interface(NMSupplicantManager *        self,
+                                       const char            *reason);
+static void     _create_iface_dbus_call_get_interface(NMSupplicantManager         *self,
                                                       NMSupplMgrCreateIfaceHandle *handle,
-                                                      const char *                 ifname);
-static void     _create_iface_dbus_call_create_interface(NMSupplicantManager *        self,
+                                                      const char                  *ifname);
+static void     _create_iface_dbus_call_create_interface(NMSupplicantManager         *self,
                                                          NMSupplMgrCreateIfaceHandle *handle,
-                                                         const char *                 ifname);
+                                                         const char                  *ifname);
 static gboolean _create_iface_fail_on_idle_cb(gpointer user_data);
 
 static gboolean _available_reset_cb(gpointer user_data);
@@ -172,8 +172,8 @@ _caps_set(NMSupplicantManagerPrivate *priv, NMSupplCapType type, NMTernary value
 
 static void
 _dbus_call_remove_interface(GDBusConnection *dbus_connection,
-                            const char *     name_owner,
-                            const char *     iface_path)
+                            const char      *name_owner,
+                            const char      *iface_path)
 {
     nm_assert(G_IS_DBUS_CONNECTION(dbus_connection));
     nm_assert(name_owner);
@@ -195,8 +195,8 @@ _dbus_call_remove_interface(GDBusConnection *dbus_connection,
 
 void
 _nm_supplicant_manager_dbus_call_remove_interface(NMSupplicantManager *self,
-                                                  const char *         name_owner,
-                                                  const char *         iface_path)
+                                                  const char          *name_owner,
+                                                  const char          *iface_path)
 {
     _dbus_call_remove_interface(NM_SUPPLICANT_MANAGER_GET_PRIVATE(self)->dbus_connection,
                                 name_owner,
@@ -208,8 +208,8 @@ _nm_supplicant_manager_dbus_call_remove_interface(NMSupplicantManager *self,
 static void
 on_supplicant_wfd_ies_set(GObject *source_object, GAsyncResult *result, gpointer user_data)
 {
-    gs_unref_variant GVariant *res = NULL;
-    gs_free_error GError *error    = NULL;
+    gs_unref_variant GVariant *res   = NULL;
+    gs_free_error GError      *error = NULL;
 
     res = g_dbus_connection_call_finish(G_DBUS_CONNECTION(source_object), result, &error);
     if (!res)
@@ -244,8 +244,7 @@ nm_supplicant_manager_set_wfd_ies(NMSupplicantManager *self, GBytes *wfd_ies)
 
     g_variant_builder_add(&params, "s", NM_WPAS_DBUS_INTERFACE);
     g_variant_builder_add(&params, "s", "WFDIEs");
-    g_variant_builder_add_value(&params,
-                                g_variant_new_variant(nm_utils_gbytes_to_variant_ay(wfd_ies)));
+    g_variant_builder_add_value(&params, g_variant_new_variant(nm_g_bytes_to_variant_ay(wfd_ies)));
 
     g_dbus_connection_call(priv->dbus_connection,
                            priv->name_owner->str,
@@ -266,10 +265,10 @@ nm_supplicant_manager_set_wfd_ies(NMSupplicantManager *self, GBytes *wfd_ies)
 static gboolean
 _poke_name_owner_timeout_cb(gpointer user_data)
 {
-    NMSupplicantManager *       self        = user_data;
-    NMSupplicantManagerPrivate *priv        = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
-    gs_free_error GError *error             = NULL;
-    gboolean              available_changed = FALSE;
+    NMSupplicantManager        *self              = user_data;
+    NMSupplicantManagerPrivate *priv              = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
+    gs_free_error GError       *error             = NULL;
+    gboolean                    available_changed = FALSE;
 
     nm_assert(!priv->name_owner);
 
@@ -308,8 +307,8 @@ _poke_name_owner_timeout_cb(gpointer user_data)
 static void
 _poke_name_owner_cb(GObject *source, GAsyncResult *result, gpointer user_data)
 {
-    gs_unref_variant GVariant *res = NULL;
-    gs_free_error GError *error    = NULL;
+    gs_unref_variant GVariant *res   = NULL;
+    gs_free_error GError      *error = NULL;
 
     res = g_dbus_connection_call_finish(G_DBUS_CONNECTION(source), result, &error);
     if (nm_utils_error_is_cancelled(error))
@@ -348,8 +347,8 @@ _poke_name_owner(NMSupplicantManager *self)
 
 static void
 _create_iface_complete(NMSupplMgrCreateIfaceHandle *handle,
-                       NMSupplicantInterface *      supp_iface,
-                       GError *                     error)
+                       NMSupplicantInterface       *supp_iface,
+                       GError                      *error)
 {
     nm_assert(!supp_iface || NM_IS_SUPPLICANT_INTERFACE(supp_iface));
     nm_assert((!!supp_iface) != (!!error));
@@ -386,13 +385,13 @@ _create_iface_complete(NMSupplMgrCreateIfaceHandle *handle,
 }
 
 static void
-_create_iface_add(NMSupplicantManager *        self,
+_create_iface_add(NMSupplicantManager         *self,
                   NMSupplMgrCreateIfaceHandle *handle,
-                  const char *                 iface_path_str,
+                  const char                  *iface_path_str,
                   gboolean                     created_by_us)
 {
-    NMSupplicantManagerPrivate *priv                  = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
-    nm_auto_ref_string NMRefString *iface_path        = NULL;
+    NMSupplicantManagerPrivate            *priv       = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
+    nm_auto_ref_string NMRefString        *iface_path = NULL;
     gs_unref_object NMSupplicantInterface *supp_iface = NULL;
 
     iface_path = nm_ref_string_new(iface_path_str);
@@ -426,13 +425,13 @@ _create_iface_add(NMSupplicantManager *        self,
 static void
 _create_iface_dbus_call_get_interface_cb(GObject *source, GAsyncResult *result, gpointer user_data)
 {
-    GDBusConnection *            dbus_connection = G_DBUS_CONNECTION(source);
+    GDBusConnection             *dbus_connection = G_DBUS_CONNECTION(source);
     NMSupplMgrCreateIfaceHandle *handle;
-    NMSupplicantManager *        self;
-    NMSupplicantManagerPrivate * priv;
-    gs_unref_variant GVariant *res = NULL;
-    gs_free_error GError *error    = NULL;
-    const char *          iface_path_str;
+    NMSupplicantManager         *self;
+    NMSupplicantManagerPrivate  *priv;
+    gs_unref_variant GVariant   *res   = NULL;
+    gs_free_error GError        *error = NULL;
+    const char                  *iface_path_str;
 
     res = g_dbus_connection_call_finish(dbus_connection, result, &error);
 
@@ -479,18 +478,18 @@ _create_iface_dbus_call_get_interface_cb(GObject *source, GAsyncResult *result,
 }
 
 static void
-_create_iface_dbus_call_create_interface_cb(GObject *     source,
+_create_iface_dbus_call_create_interface_cb(GObject      *source,
                                             GAsyncResult *result,
                                             gpointer      user_data)
 {
-    GDBusConnection *            dbus_connection = G_DBUS_CONNECTION(source);
+    GDBusConnection             *dbus_connection = G_DBUS_CONNECTION(source);
     NMSupplMgrCreateIfaceHandle *handle          = user_data;
-    NMSupplicantManager *        self;
-    NMSupplicantManagerPrivate * priv;
-    gs_unref_variant GVariant *res = NULL;
-    gs_free_error GError *error    = NULL;
-    const char *          iface_path_str;
-    char                  ifname[NMP_IFNAMSIZ];
+    NMSupplicantManager         *self;
+    NMSupplicantManagerPrivate  *priv;
+    gs_unref_variant GVariant   *res   = NULL;
+    gs_free_error GError        *error = NULL;
+    const char                  *iface_path_str;
+    char                         ifname[NMP_IFNAMSIZ];
 
     res = g_dbus_connection_call_finish(dbus_connection, result, &error);
 
@@ -554,9 +553,9 @@ _create_iface_dbus_call_create_interface_cb(GObject *     source,
 }
 
 static void
-_create_iface_dbus_call_get_interface(NMSupplicantManager *        self,
+_create_iface_dbus_call_get_interface(NMSupplicantManager         *self,
                                       NMSupplMgrCreateIfaceHandle *handle,
-                                      const char *                 ifname)
+                                      const char                  *ifname)
 {
     NMSupplicantManagerPrivate *priv = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
 
@@ -578,9 +577,9 @@ _create_iface_dbus_call_get_interface(NMSupplicantManager *        self,
 }
 
 static void
-_create_iface_dbus_call_create_interface(NMSupplicantManager *        self,
+_create_iface_dbus_call_create_interface(NMSupplicantManager         *self,
                                          NMSupplMgrCreateIfaceHandle *handle,
-                                         const char *                 ifname)
+                                         const char                  *ifname)
 {
     NMSupplicantManagerPrivate *priv = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
     GVariantBuilder             builder;
@@ -677,13 +676,13 @@ _create_iface_fail_on_idle_cb(gpointer user_data)
 }
 
 NMSupplMgrCreateIfaceHandle *
-nm_supplicant_manager_create_interface(NMSupplicantManager *                self,
+nm_supplicant_manager_create_interface(NMSupplicantManager                 *self,
                                        int                                  ifindex,
                                        NMSupplicantDriver                   driver,
                                        NMSupplicantManagerCreateInterfaceCb callback,
                                        gpointer                             user_data)
 {
-    NMSupplicantManagerPrivate * priv;
+    NMSupplicantManagerPrivate  *priv;
     NMSupplMgrCreateIfaceHandle *handle;
 
     g_return_val_if_fail(NM_IS_SUPPLICANT_MANAGER(self), NULL);
@@ -752,7 +751,7 @@ nm_supplicant_manager_create_interface(NMSupplicantManager *                self
 static void
 _create_iface_proceed_all(NMSupplicantManager *self, GError *error)
 {
-    NMSupplicantManagerPrivate * priv = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
+    NMSupplicantManagerPrivate  *priv = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
     NMSupplMgrCreateIfaceHandle *handle;
 
     nm_assert(error || priv->name_owner);
@@ -811,8 +810,8 @@ nm_supplicant_manager_create_interface_cancel(NMSupplMgrCreateIfaceHandle *handl
 NMSupplicantInterface *
 nm_supplicant_manager_create_interface_from_path(NMSupplicantManager *self, const char *object_path)
 {
-    NMSupplicantManagerPrivate *priv;
-    NMSupplicantInterface *     supp_iface;
+    NMSupplicantManagerPrivate     *priv;
+    NMSupplicantInterface          *supp_iface;
     nm_auto_ref_string NMRefString *iface_path = NULL;
 
     g_return_val_if_fail(NM_IS_SUPPLICANT_MANAGER(self), NULL);
@@ -838,17 +837,17 @@ nm_supplicant_manager_create_interface_from_path(NMSupplicantManager *self, cons
 
 static void
 _dbus_interface_removed_cb(GDBusConnection *connection,
-                           const char *     sender_name,
-                           const char *     object_path,
-                           const char *     signal_interface_name,
-                           const char *     signal_name,
-                           GVariant *       parameters,
+                           const char      *sender_name,
+                           const char      *object_path,
+                           const char      *signal_interface_name,
+                           const char      *signal_name,
+                           GVariant        *parameters,
                            gpointer         user_data)
 {
-    NMSupplicantManager *       self = user_data;
-    NMSupplicantManagerPrivate *priv = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
-    NMSupplicantInterface *     supp_iface;
-    const char *                iface_path_str;
+    NMSupplicantManager            *self = user_data;
+    NMSupplicantManagerPrivate     *priv = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
+    NMSupplicantInterface          *supp_iface;
+    const char                     *iface_path_str;
     nm_auto_ref_string NMRefString *iface_path = NULL;
 
     nm_assert(nm_streq(sender_name, priv->name_owner->str));
@@ -872,7 +871,7 @@ _dbus_interface_removed_cb(GDBusConnection *connection,
 static void
 _dbus_get_capabilities_cb(GVariant *res, GError *error, gpointer user_data)
 {
-    NMSupplicantManager *       self;
+    NMSupplicantManager        *self;
     NMSupplicantManagerPrivate *priv;
 
     if (nm_utils_error_is_cancelled(error))
@@ -906,15 +905,15 @@ _dbus_get_capabilities_cb(GVariant *res, GError *error, gpointer user_data)
 
     if (res) {
         nm_auto_free_variant_iter GVariantIter *res_iter = NULL;
-        const char *                            res_key;
-        GVariant *                              res_val;
+        const char                             *res_key;
+        GVariant                               *res_val;
 
         g_variant_get(res, "(a{sv})", &res_iter);
         while (g_variant_iter_loop(res_iter, "{&sv}", &res_key, &res_val)) {
             if (nm_streq(res_key, "Capabilities")) {
                 if (g_variant_is_of_type(res_val, G_VARIANT_TYPE_STRING_ARRAY)) {
                     gs_free const char **array = NULL;
-                    const char **        a;
+                    const char         **a;
 
                     array = g_variant_get_strv(res_val, NULL);
                     _caps_set(priv, NM_SUPPL_CAP_TYPE_AP, NM_TERNARY_FALSE);
@@ -963,7 +962,7 @@ _dbus_get_capabilities_cb(GVariant *res, GError *error, gpointer user_data)
             if (nm_streq(res_key, "EapMethods")) {
                 if (g_variant_is_of_type(res_val, G_VARIANT_TYPE_STRING_ARRAY)) {
                     gs_free const char **array = NULL;
-                    const char **        a;
+                    const char         **a;
 
                     array = g_variant_get_strv(res_val, NULL);
                     if (array) {
@@ -994,6 +993,7 @@ _dbus_get_capabilities_cb(GVariant *res, GError *error, gpointer user_data)
           " MESH%c"
           " FAST%c"
           " WFD%c"
+          " SUITEB192%c"
           "",
           NM_SUPPL_CAP_TO_CHAR(priv->capabilities, NM_SUPPL_CAP_TYPE_AP),
           NM_SUPPL_CAP_TO_CHAR(priv->capabilities, NM_SUPPL_CAP_TYPE_PMF),
@@ -1003,7 +1003,8 @@ _dbus_get_capabilities_cb(GVariant *res, GError *error, gpointer user_data)
           NM_SUPPL_CAP_TO_CHAR(priv->capabilities, NM_SUPPL_CAP_TYPE_SHA384),
           NM_SUPPL_CAP_TO_CHAR(priv->capabilities, NM_SUPPL_CAP_TYPE_MESH),
           NM_SUPPL_CAP_TO_CHAR(priv->capabilities, NM_SUPPL_CAP_TYPE_FAST),
-          NM_SUPPL_CAP_TO_CHAR(priv->capabilities, NM_SUPPL_CAP_TYPE_WFD));
+          NM_SUPPL_CAP_TO_CHAR(priv->capabilities, NM_SUPPL_CAP_TYPE_WFD),
+          NM_SUPPL_CAP_TO_CHAR(priv->capabilities, NM_SUPPL_CAP_TYPE_SUITEB192));
 
     nm_assert(g_hash_table_size(priv->supp_ifaces) == 0);
     nm_assert(c_list_is_empty(&priv->supp_lst_head));
@@ -1014,7 +1015,7 @@ _dbus_get_capabilities_cb(GVariant *res, GError *error, gpointer user_data)
 /*****************************************************************************/
 
 void
-_nm_supplicant_manager_unregister_interface(NMSupplicantManager *  self,
+_nm_supplicant_manager_unregister_interface(NMSupplicantManager   *self,
                                             NMSupplicantInterface *supp_iface)
 {
     NMSupplicantManagerPrivate *priv = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
@@ -1030,8 +1031,8 @@ _nm_supplicant_manager_unregister_interface(NMSupplicantManager *  self,
 }
 
 static void
-_supp_iface_add(NMSupplicantManager *  self,
-                NMRefString *          iface_path,
+_supp_iface_add(NMSupplicantManager   *self,
+                NMRefString           *iface_path,
                 NMSupplicantInterface *supp_iface)
 {
     NMSupplicantManagerPrivate *priv = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
@@ -1042,10 +1043,10 @@ _supp_iface_add(NMSupplicantManager *  self,
 }
 
 static void
-_supp_iface_remove_one(NMSupplicantManager *  self,
+_supp_iface_remove_one(NMSupplicantManager   *self,
                        NMSupplicantInterface *supp_iface,
                        gboolean               force_remove_from_supplicant,
-                       const char *           reason)
+                       const char            *reason)
 {
 #if NM_MORE_ASSERTS
     _nm_unused gs_unref_object NMSupplicantInterface *supp_iface_keep_alive =
@@ -1065,10 +1066,10 @@ _supp_iface_remove_one(NMSupplicantManager *  self,
 static void
 _supp_iface_remove_all(NMSupplicantManager *self,
                        gboolean             force_remove_from_supplicant,
-                       const char *         reason)
+                       const char          *reason)
 {
     NMSupplicantManagerPrivate *priv = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
-    NMSupplicantInterface *     supp_iface;
+    NMSupplicantInterface      *supp_iface;
 
     while ((supp_iface = c_list_first_entry(&priv->supp_lst_head, NMSupplicantInterface, supp_lst)))
         _supp_iface_remove_one(self, supp_iface, force_remove_from_supplicant, reason);
@@ -1079,7 +1080,7 @@ _supp_iface_remove_all(NMSupplicantManager *self,
 static gboolean
 _available_reset_cb(gpointer user_data)
 {
-    NMSupplicantManager *       self = user_data;
+    NMSupplicantManager        *self = user_data;
     NMSupplicantManagerPrivate *priv = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
 
     priv->available_reset_id = 0;
@@ -1210,16 +1211,16 @@ name_owner_changed(NMSupplicantManager *self, const char *name_owner, gboolean f
 
 static void
 name_owner_changed_cb(GDBusConnection *connection,
-                      const char *     sender_name,
-                      const char *     object_path,
-                      const char *     interface_name,
-                      const char *     signal_name,
-                      GVariant *       parameters,
+                      const char      *sender_name,
+                      const char      *object_path,
+                      const char      *interface_name,
+                      const char      *signal_name,
+                      GVariant        *parameters,
                       gpointer         user_data)
 {
     gs_unref_object NMSupplicantManager *self = g_object_ref(user_data);
-    NMSupplicantManagerPrivate *         priv = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
-    const char *                         name_owner;
+    NMSupplicantManagerPrivate          *priv = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
+    const char                          *name_owner;
 
     if (!g_variant_is_of_type(parameters, G_VARIANT_TYPE("(sss)")))
         return;
@@ -1245,7 +1246,7 @@ name_owner_changed_cb(GDBusConnection *connection,
 static void
 get_name_owner_cb(const char *name_owner, GError *error, gpointer user_data)
 {
-    NMSupplicantManager *       self = user_data;
+    NMSupplicantManager        *self = user_data;
     NMSupplicantManagerPrivate *priv = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
 
     if (!name_owner && nm_utils_error_is_cancelled(error))
@@ -1298,7 +1299,7 @@ nm_supplicant_manager_init(NMSupplicantManager *self)
 static void
 dispose(GObject *object)
 {
-    NMSupplicantManager *       self = (NMSupplicantManager *) object;
+    NMSupplicantManager        *self = (NMSupplicantManager *) object;
     NMSupplicantManagerPrivate *priv = NM_SUPPLICANT_MANAGER_GET_PRIVATE(self);
 
     _supp_iface_remove_all(self, TRUE, "NMSupplicantManager is disposing");
