@@ -11,6 +11,7 @@
 #include <fcntl.h>
 
 #include "NetworkManagerUtils.h"
+#include "libnm-platform/nm-platform-utils.h"
 
 #include "nm-test-utils-core.h"
 
@@ -97,7 +98,7 @@ test_nm_utils_kill_child_async_do(const char *name,
                                   int         sig,
                                   guint32     wait_before_kill_msec,
                                   gboolean    expected_success,
-                                  const int * expected_child_status)
+                                  const int  *expected_child_status)
 {
     gboolean                                   success;
     struct test_nm_utils_kill_child_async_data data = {};
@@ -134,7 +135,7 @@ test_nm_utils_kill_child_sync_do(const char *name,
                                  int         sig,
                                  guint32     wait_before_kill_msec,
                                  gboolean    expected_success,
-                                 const int * expected_child_status)
+                                 const int  *expected_child_status)
 {
     gboolean success;
     int      child_status = -1;
@@ -225,11 +226,11 @@ static void
 do_test_nm_utils_kill_child(void)
 {
     GLogLevelFlags fatal_mask;
-    char *         argv_watchdog[] = {
+    char          *argv_watchdog[] = {
         "bash",
         "-c",
         "sleep 4; "
-        "kill -KILL 0; #watchdog for #" TEST_TOKEN,
+                 "kill -KILL 0; #watchdog for #" TEST_TOKEN,
         NULL,
     };
     char *argv1[] = {
@@ -463,7 +464,7 @@ test_nm_utils_kill_child(void)
 static void
 _remove_at_indexes_init_random_idx(GArray *idx, guint array_len, guint idx_len)
 {
-    GRand *       rand = nmtst_get_rand();
+    GRand        *rand = nmtst_get_rand();
     gs_free char *mask = NULL;
     guint         i, max_test_idx;
 
@@ -514,7 +515,7 @@ _remove_at_indexes_init_random_idx(GArray *idx, guint array_len, guint idx_len)
 static void
 test_nm_utils_array_remove_at_indexes(void)
 {
-    gs_unref_array GArray *idx = NULL, *array = NULL;
+    gs_unref_array GArray         *idx = NULL, *array = NULL;
     gs_unref_hashtable GHashTable *unique = NULL;
     guint                          i_len, i_idx_len, i_rnd, i;
 
@@ -586,7 +587,7 @@ test_nm_ethernet_address_is_valid(void)
 /*****************************************************************************/
 
 static void
-test_nm_utils_new_vlan_name(void)
+test_nmp_utils_new_vlan_name(void)
 {
     guint       i, j;
     const char *parent_names[] = {
@@ -614,7 +615,7 @@ test_nm_utils_new_vlan_name(void)
 
             vlan_id_s = g_strdup_printf(".%d", vlan_id);
 
-            ifname = nm_utils_new_vlan_name(parent_names[i], vlan_id);
+            ifname = nmp_utils_new_vlan_name(parent_names[i], vlan_id);
             g_assert(ifname && ifname[0]);
             g_assert_cmpint(strlen(ifname),
                             ==,
@@ -643,7 +644,7 @@ main(int argc, char **argv)
     g_test_add_func("/general/nm_utils_array_remove_at_indexes",
                     test_nm_utils_array_remove_at_indexes);
     g_test_add_func("/general/nm_ethernet_address_is_valid", test_nm_ethernet_address_is_valid);
-    g_test_add_func("/general/nm_utils_new_vlan_name", test_nm_utils_new_vlan_name);
+    g_test_add_func("/general/nmp_utils_new_vlan_name", test_nmp_utils_new_vlan_name);
 
     return g_test_run();
 }
